Package: MD2sample
Title: Various Methods for the Two Sample Problem in D>1 Dimensions
Version: 1.3.0
Authors@R: 
    person("Wolfgang", "Rolke", , "wolfgang.rolke@upr.edu", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3514-726X"))
Description: The routine twosample_test() in this package runs the
    two-sample test using various test statistic for multivariate data. 
    The user can also run several tests and then find a p value adjusted for simultaneous inference.
    The p values are found via permutation or via the parametric bootstrap. 
    The routine twosample_power() allows the estimation of the power of the tests. 
    The routine run.studies() allows a user to quickly study the power of a new method and 
    how it compares to those included in the package.
    For details of the methods and references see the included vignettes.
License: GPL (>= 2)
Encoding: UTF-8
LinkingTo: Rcpp
Imports: Rcpp, parallel, stats, FNN, copula, ade4, gTests, igraph, lsa,
        microbenchmark, mvtnorm, Ball
Suggests: rmarkdown, knitr, ggplot2, egg, testthat (>= 3.0.0)
VignetteBuilder: knitr
Depends: R (>= 3.5)
LazyData: true
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-08-02 13:05:54 UTC; Wolfgang
Author: Wolfgang Rolke [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3514-726X>)
Maintainer: Wolfgang Rolke <wolfgang.rolke@upr.edu>
Repository: CRAN
Date/Publication: 2026-08-04 10:30:40 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-08-04 12:53:57 UTC; unix
Archs: MD2sample.so.dSYM

# Generated by roxygen2: do not edit by hand

export(FR.test)
export(TS_cont_pval)
export(bincounterR)
export(case.studies)
export(change.marginals)
export(chiTS.cont)
export(chiTS.disc)
export(power_pvals)
export(rDalitz)
export(run.studies)
export(signif_digits)
export(simTS)
export(simpvals)
export(test_methods)
export(timecheck)
export(twosample_power)
export(twosample_test)
export(twosample_test_adjusted_pvalue)
importFrom(Rcpp,sourceCpp)
importFrom(stats,qexp)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rexp)
importFrom(stats,runif)
useDynLib(MD2sample, .registration=TRUE)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(MD2sample)
cases=MD2sample::case.studies(ReturnCaseNames=TRUE)

## -----------------------------------------------------------------------------
multiple.graphs = function (px, py,xname=" ") {
  egg::ggarrange(px, py, ncol = 2, nrow = 1)
} 

## -----------------------------------------------------------------------------
for(i in seq_along(cases)) {
   if(!endsWith(cases[i], "D2")) next
   tmp=case.studies(i, 2000)
   print(cases[i])
   p=tmp$param_alt[2]
   if(startsWith(cases[i],"Dalitz")) p=10*p
   dta=tmp$f(p)
   dtax=data.frame(x=dta$x[,1], y=dta$x[,2])
   dtay=data.frame(x=dta$y[,1], y=dta$y[,2])
   px=ggplot2::ggplot(dtax, ggplot2::aes(x=x, y=y))+
     ggplot2::geom_point(color="blue", alpha=0.5, size=0.5)+
     ggplot2::labs(x=expression(x[1]), y=expression(x[2])) 
     
   py=ggplot2::ggplot(dtay, ggplot2::aes(x=x, y=y))+
     ggplot2::geom_point(color="red", alpha=0.5)+
     ggplot2::labs(x=expression(x[1]), y=expression(x[2])) 
  multiple.graphs(px, py, cases[i])
}

## -----------------------------------------------------------------------------
for(i in seq_along(cases)) {
   if(!endsWith(cases[i], "M")) next
   tmp=case.studies(i, 2000)
   print(cases[i])
   p=tmp$param_alt[2]
   if(startsWith(cases[i],"Dalitz")) p=10*p
   dta=tmp$f(p)
   dtax=data.frame(x=dta$x[,1], y=dta$x[,2])
   dtay=data.frame(x=dta$y[,1], y=dta$y[,2])
   px=ggplot2::ggplot(dtax, ggplot2::aes(x=x, y=y))+
     ggplot2::geom_point(color="blue", alpha=0.5, size=0.5)+
     ggplot2::labs(x=expression(x[1]), y=expression(x[2])) 
     
   py=ggplot2::ggplot(dtay, ggplot2::aes(x=x, y=y))+
     ggplot2::geom_point(color="red", alpha=0.5)+
     ggplot2::labs(x=expression(x[1]), y=expression(x[2])) 
  multiple.graphs(px, py, cases[i])
}

#' This function does the tests by Chen and Friedman
#' @param x  a matrix of numbers 
#' @param y a matrix of numbers 
#' @return a list with statistics and p values
edge.tests=function(x,y) {
   nx=nrow(x)
   ny=nrow(y)
   dxy=as.matrix(stats::dist(rbind(x,y)))
   counts=matrix(1, nx+ny,2)
   tmp=gTests::g.tests(gTests::getGraph(counts,dxy, 1),1:nx, (nx+1):(nx+ny))
   out=list(statistics=unlist(tmp)[c(1,3,5,7)],
            p.values=unlist(tmp)[c(2,4,6,8)])
   names(out$statistics)=paste0("CF",1:4)
   names(out$p.values)=paste0("CF",1:4)
   out
}

#' This function does the chi square two sample test for continuous data in two dimensions
#' @param dta a list with values and counts 
#' @param minexpcount =5 minimum counts required per bin
#' @return a list with statistics and p values
chisq2D_test_disc=function(dta, minexpcount=5) {
  if(is.list(dta))
     dta=cbind(dta[["vals_x"]], dta[["vals_y"]], dta[["x"]], dta[["y"]])
  colnames(dta)=c("vals_x", "vals_y", "x", "y")
  tmp=0
  vx=sort(unique(dta[,1]))
  vy=sort(unique(dta[,2]))
  nbins=c(length(vx),length(vy))
  out=list(statistics=tmp, p.values=tmp, df=tmp)
  O_xy=matrix(0, nbins[1], nbins[2])
  dimnames(O_xy)=list(vx, vy)
  O_x=O_xy
  O_y=O_xy
  for(i in seq_along(vx)) {
    for(j in seq_along(vy)) {
      a=dta[((dta[,1]==vx[i])&(dta[,2]==vy[j])), ]
      O_x[i,j]=a[3]
      O_y[i,j]=a[4]
      O_xy[i,j]=O_x[i,j]+O_y[i,j]
    }
  }
  repeat {
       if(min(O_xy, na.rm=TRUE)>=minexpcount) break
       lowest = c(1, 1)
       tmp = max(O_xy, na.rm = TRUE)
       for(i in 1:nbins[1]) {
         for(j in 1:nbins[2]) {
           if(!is.na(O_xy[i,j]) && O_xy[i,j]<tmp) {
             tmp=O_xy[i,j]
             lowest=c(i,j)
           }
         }
       }
       tmp=max(O_xy, na.rm = TRUE)
       nearest=c(-1,-1)
       for(step in 1:max(nbins)) {
         I=lowest[1]+c(-step:step)
         I=I[1<=I&I<=nbins[1]]
         J=lowest[2]+c(-step:step)
         J=J[1<=J&J<=nbins[2]]
         for(i in I) {
           for(j in J) {
             if(i==lowest[1]&&j==lowest[2]) next
             if(!is.na(O_xy[i,j])&&O_xy[i,j]<tmp) {
               tmp=O_xy[i,j]
               nearest = c(i,j)
             }
           }
         }
         if(nearest[1]>0) break
       }
       O_x[lowest[1], lowest[2]] = O_x[lowest[1], lowest[2]] + 
                                   O_x[nearest[1], nearest[2]]
       O_x[nearest[1], nearest[2]] = NA
       O_y[lowest[1], lowest[2]] = O_y[lowest[1], lowest[2]] + 
         O_y[nearest[1], nearest[2]]
       O_y[nearest[1], nearest[2]] = NA
       O_xy[lowest[1], lowest[2]] = O_xy[lowest[1], lowest[2]] + 
         O_xy[nearest[1], nearest[2]]
       O_xy[nearest[1], nearest[2]] = NA
       step=1
  }
  O_x=O_x[!is.na(O_x)]
  O_y=O_y[!is.na(O_y)]
  O_xy=O_xy[!is.na(O_xy)]
  s=sqrt(sum(O_x)/sum(O_y))
  chi=sum((O_x/s-s*O_y)^2/O_xy)
  names(chi)="ChiSquare"
  df=length(O_x)-1
  names(df)="ChiSquare"
  out[[1]]=chi
  p=1-stats::pchisq(chi,df)
  names(p)="ChiSquare"  
  out[[2]]=p
  out[[3]]=df
  out
}

#' This function creates data for Dalitz plot
#' @param nyadd =0
#' @param which type of Daliz plot.
#' @param addon what to add to uniform
#' @param n =200, sample size of both data sets.
#' @param nx =n, sample size of x data set.
#' @param ny =n, sample size of y data set.
#' @keywords internal
#' @return a list of functions
#' @export
rDalitz=function(nyadd=0, which=1, addon, n=200, nx=n, ny=n) {
  
  if(which==1) {M=1.97;m1=0.4937;m2=0.4937;m3=0.1396;delta=0.3}
  if(which==2)  {M=3;m1=0.1396;m2=0.4976;m3=0.938;delta=0.2}
  if(which==3)  {M=3;m1=0.3099;m2=0.3101;m3=0.3094;delta=0.2}
  sq=function(x) {
     y=rep(NA, length(x))
     for(i in seq_along(x)) if(x[i]>=0) y[i]=sqrt(x[i])
     y
  }
  doCut=function() {
    E2=(m12s-m1^2+m2^2)/(2*sqrt(m12s))
    E3=(M^2-m12s-m3^2+m2^2)/(2*sqrt(m12s))
    m23max=(E2+E3)^2-(sq(E2^2-m2^2)-sq(E3^2-m3^2))^2
    m23min=(E2+E3)^2-(sq(E2^2-m2^2)+sq(E3^2-m3^2))^2
    u=cbind(m12s, m23s)
    u=u[u[,2]>m23min & u[,2]<m23max, ]
    u=u[!is.na(u[,1]),]
    u[!is.na(u[,2]),]
  }
  # Find border    
  xrange=c( (m1+m2)^2,(M-m3)^2)+delta*c(-1,1)
  yrange=c( (m2+m3)^2,(M-m1)^2)+delta*c(-1,1)    
  m12s=seq(xrange[1], xrange[2], length=3000)
  E2=(m12s-m1^2+m2^2)/(2*sqrt(m12s))
  E3=(M^2-m12s-m3^2+m2^2)/(2*sqrt(m12s))
  m23max=(E2+E3)^2-(sq(E2^2-m2^2)-sq(E3^2-m3^2))^2
  m23min=(E2+E3)^2-(sq(E2^2-m2^2)+sq(E3^2-m3^2))^2
  borders=cbind(m12s, m23min, m23max)
  borders=borders[!is.na(borders[,2]), ]
  borders=borders[!is.na(borders[,3]), ]
  mbor=nrow(borders)
  # Find uniform data
  u=NULL 
  repeat {
      m12s=runif(nx+ny, xrange[1], xrange[2])
      m23s=runif(nx+ny, yrange[1], yrange[2])
      u=rbind(u, doCut())
      if(nrow(u)>nx+ny) break
  }
  x=u[1:nx,]
  y=u[(1+nx):(nx+ny),]
  if(nyadd==0) return(list(x=x, y=y, borders=borders))
  add_data=as.list(1:4)
  names(add_data)=c("Left Stripe", "Right Stripe",
                    "Bottom Stripe", "Diagonal Stripe")
# left stripe    
    m12s=runif(ny, borders[1,1], borders[100,1])
    E2=(m12s-m1^2+m2^2)/(2*sqrt(m12s))
    E3=(M^2-m12s-m3^2+m2^2)/(2*sqrt(m12s))
    m23max=(E2+E3)^2-(sq(E2^2-m2^2)-sq(E3^2-m3^2))^2
    m23min=(E2+E3)^2-(sq(E2^2-m2^2)+sq(E3^2-m3^2))^2 
    m23s=m23min+rbeta(ny, 0.1, 0.1)*(m23max-m23min)
    add_data[[1]]=cbind(m12s, m23s)
#bottom stripe  
    m12s=sort(runif(ny, borders[1,1], borders[0.5*mbor,1]))
    ab=c(borders[0.6*mbor, 2], borders[0.1*mbor,2])
    q=(m12s-min(m12s))/(max(m12s)-min(m12s))/2.3
    m23s=runif(ny, ab[1]+q*diff(ab),ab[2]-q*diff(ab))
    add_data[[3]]=doCut()
#right stripe    
    tmp=rexp(2*ny,3)
    tmp=1-(tmp[tmp<1])[1:ny]
    m12s=sort(runif(ny, borders[0.6*mbor,1], borders[mbor,1]))
    m12s=borders[0.6*mbor,1]+(borders[mbor,1]-borders[0.6*mbor,1])*tmp
    ab=c(borders[0.3*mbor, 2], borders[0.1*mbor,2])
    q=(m12s-min(m12s))/(max(m12s)-min(m12s))/2.5
    q=max(q)-q
    m23s=runif(ny, ab[1]+q*diff(ab),ab[2]-q*diff(ab))
    add_data[[2]]=doCut()
#Diagonal
  m12s=runif(3*nyadd, borders[1,1],borders[mbor,1])
  q=c(range(borders[,1]),range(borders[,2:3]))
  lne=function(x) (q[4]-q[3])/(q[1]-q[2])*(x-q[1])+q[4]
  m23s=lne(m12s)+runif(3*nyadd, -0.05, 0.05)
  add_data[[4]]=doCut()
  k=length(addon)
  y=y[1:(ny-nyadd*k), ]
  for(i in addon) y=rbind(y, add_data[[i]][1:nyadd,])
  list(x=x, y=y, borders=borders)
}

#' Power Estimation for Multivariate Two-Sample Tests
#' 
#' Estimate the power of various two sample tests using Rcpp and parallel computing.
#' 
#' For details consult vignette("MD2sample","MD2sample")
#' 
#' @param  f  function to generate a list with data sets x and y for continuous data or
#'         a matrix with columns vals_x, vals_y, x and y for discrete data.
#' @param  ... additional arguments passed to f, up to 2.
#' @param  TS routine to calculate test statistics for new tests.
#' @param  TSextra additional info passed to TS, if necessary.
#' @param  alpha =0.05, the type I error probability of the hypothesis test. 
#' @param  B =1000, number of simulation runs.
#' @param  nbins =c(5, 5), number of bins for chi square test if Dim=2.
#' @param  minexpcount =5, lowest required count for chi-square test.
#' @param  Ranges =matrix(c(-Inf, Inf, -Inf, Inf),2,2), a 2x2 matrix with lower and upper bounds.
#' @param  samplingmethod ="Binomial" for Binomial sampling or "independence" for independence 
#'         sampling in the discrete data case.
#' @param  rnull function to generate new data sets for parametric bootstrap.
#' @param  With.p.value =FALSE, does user supplied routine return p values?
#' @param  DoTransform =TRUE, should data be transformed to  to unit hypercube? 
#' @param  SuppressMessages =FALSE, should messages be printed?
#' @param  LargeSampleOnly =FALSE, should only methods with large sample theories be run?
#' @param  maxProcessor number of cores to use. If missing the number of physical cores-1 
#'             is used. If set to 1 no parallel processing is done.
#' @param  doMethods ="all", which methods should be included?
#' @return A numeric matrix or vector of power values.
#' @examples
#' #Note that the resulting power estimates are meaningless because
#' #of the extremely low number of simulation runs B, required because of CRAN timing rule
#' #
#' #Power of tests when one data set comes from a standard normal multivariate distribution function
#' #and the other data set from a multivariate normal with correlation
#' #number of simulation runs is ridiculously small because of CRAN submission rules
#' f=function(a=0) {
#'  S=diag(2) 
#'  x=mvtnorm::rmvnorm(100, sigma = S)
#'  S[1,2]=a
#'  S[2,1]=a
#'  y=mvtnorm::rmvnorm(120, sigma = S)
#'  list(x=x, y=y)
#' }
#' twosample_power(f, c(0, 0.5), B=10, maxProcessor=1)
#' #Power of use supplied test. Example is a (included) chi-square test:
#' TSextra=list(which="statistics", nbins=rbind(c(3,3), c(4,4)))
#' twosample_power(f, c(0, 0.5), TS=chiTS.cont, TSextra=TSextra, B=10, maxProcessor=1)
#' #Same example, but this time the user supplied routine calculates p values:
#' TSextra=list(which="pvalues", nbins=c(4,4))
#' twosample_power(f, c(0, 0.5), TS=chiTS.cont, TSextra=TSextra, B=10, 
#'              With.p.value=TRUE, maxProcessor=1)
#' #Example for discrete data
#' g=function(p1, p2) {
#'   x = table(sample(1:4, size=1000, replace = TRUE))
#'   y = table(sample(1:4, size=500, replace = TRUE, prob=c(p1,p2,1,1)))
#'   cbind(vals_x=rep(1:2,2),  vals_y=rep(1:2, each=2), x=x, y=y)
#' }  
#' twosample_power(g, 1.5, 1.6, B=10, maxProcessor=1)
#' @export 
twosample_power=function(f, ..., TS, TSextra, alpha=0.05, B=1000, 
            nbins=c(5,5), minexpcount =5, Ranges=matrix(c(-Inf, Inf, -Inf, Inf),2,2),
            samplingmethod="Binomial", rnull, With.p.value=FALSE,
            DoTransform=TRUE, SuppressMessages=FALSE, 
            LargeSampleOnly=FALSE, maxProcessor, doMethods ="all") {
    if(!is.numeric(samplingmethod))  
      samplingmethod=ifelse(samplingmethod=="independence", 1, 2)
# create function rxy which generates data, with two arguments                       
    if(length(list(...))==0) { # f has 0 arguments
       rxy=function(a=0, b=0) f()
       avals=0
       bvals=0
    }
    if(length(list(...))==1) { # f has 1 argument
       rxy=function(a=0, b=0) f(a)
       avals=list(...)[[1]]
       bvals=0
    }
    if(length(list(...))==2) { # f has 2 arguments
       rxy=function(a=0, b=0) f(a,b)
       avals=list(...)[[1]]
       bvals=list(...)[[2]]
    }
  # check that avals and bvals have the same length. 
  # If they do, matrix of powers is returned without row names.
  # If one of them is a scalar, make it the same length as the other and use those
  # values as row names
  if(length(avals)!=length(bvals)) {
    if(min(c(length(avals),length(bvals)))>1) {
      if(!SuppressMessages) message("lengths of parameter vectors not compatible!\n")
      return(NULL)
    }
    if(length(avals)==1) {
      avals=rep(avals, length(bvals))
    }    
    else bvals=rep(bvals, length(avals))
  }    
# generate one data set as an example, do some setup 
  dta = rxy(avals[1], bvals[1])
  Continuous=TRUE
  if(is.matrix(dta)) { #Discrete Data
    dta=list(x=dta[,3], y=dta[,4], vals_x=dta[,1], vals_y=dta[,2])
    x=matrix(1:4,2,2) #just some dummy numbers
    y=matrix(1:4,2,2)
    Continuous=FALSE
    DoTransform=FALSE
  }
  if(test_methods(doMethods, Continuous)) return(NULL)
  if(Continuous) {
    x=dta$x
    y=dta$y
    Dim=ncol(x) # dimension of data
    if(nrow(y)<nrow(x)) { 
      if(!SuppressMessages) message("sample size of x should not be larger than sample size of y")
      return(NULL)
    }
  }  
  if(DoTransform) {
    dta=transform01(dta)
    x=dta$x
    y=dta$y
    Ranges=matrix(c(0, 1, 0, 1),2,2)
  }
  if(missing(TSextra)) TSextra=list(aaa=0)
  if(Continuous)
     TSextra = c(TSextra, 
               knn=function(x) FNN::get.knn(x, 5)$nn.index,
               dist=function(dta) find_dist(dta),
               distances=list(find_dist(list(x=x,y=y))),
               DoTransform=DoTransform,
               ParametricBootstrap=FALSE)
  else TSextra = c(TSextra, 
               dist=function(dta) NULL,
               organize=function(dta) dta=dta[order(dta[,1], dta[,2]), ],
               samplingmethod=samplingmethod,
               ParametricBootstrap=FALSE)
  if(!missing(rnull)) {
     TSextra$ParametricBootstrap=TRUE
     TSextra=c(TSextra, rnull=rnull, rawdta=list(dta))
  }   
  pwrchi=NULL # no chi square test
  if(missing(TS)) { # do included methods
    CustomTS=FALSE
    if(Continuous) { # Continuous Data
        typeTS=1
        TS=TS_cont
    }
    else { # Discrete Data
        typeTS=4
        TS=TS_disc
    }  
  }
  else { # do user-supplied tests
    CustomTS=TRUE
    if(Continuous) typeTS=length(formals(TS))
    else typeTS=ifelse(length(formals(TS))==5, 6, 5)
  }
  TS_data=calcTS(dta, TS, typeTS, TSextra)
  if(is.null(names(TS_data))) {
    if(!SuppressMessages) message("output of TS routine has to be a named vector!")
    return(NULL)
  }  
  methodnames=names(TS_data)
  
# Do a time check for power
  if(With.p.value) maxProcessor=1
  if(missing(maxProcessor)) {
    ncores=max(parallel::detectCores(logical=FALSE)-1,1)
    tm=timecheck(dta, TS, typeTS, TSextra)
    if(length(tm)==1) tm=c(tm,0)
    totaltime=2*tm*length(avals)*B
    if(max(totaltime)<20 | B<=2*ncores) 
       if(!SuppressMessages) message("maxProcessor set to 1 for faster computation")
    else if(!SuppressMessages) message(paste("Using ", ncores," cores..")) 
    maxProcessor1=1
    if(totaltime[1]>20 & B>2*ncores) 
       maxProcessor1=ncores
    maxProcessor2=1
    if(typeTS==1 & totaltime[2]>20 & B>2*ncores) 
       maxProcessor2=ncores
    if(max(totaltime)>20 & B>2*ncores) {
       if(maxProcessor1==1 & maxProcessor2>1)
          totaltime=30+totaltime[2]/maxProcessor2
       if(maxProcessor1>1 & maxProcessor2==1)
          totaltime=30+totaltime[1]/maxProcessor1
       if(maxProcessor1>1 & maxProcessor2>1)
          totaltime=50+sum(totaltime)/maxProcessor1
       totaltime=round(totaltime,-1)
       timeunit="seconds"
       if(max(totaltime)>60) {
         totaltime=round(totaltime/60,1)
         timeunit="minutes"
       }
       if(!SuppressMessages) message(paste("estimated time:", totaltime , timeunit))
    }
  }  
  else {
    maxProcessor1=maxProcessor
    maxProcessor2=maxProcessor
  }
# Run power routine for new test which returns p value(s)
  if(With.p.value) {
    pwr=power_pvals(rxy, avals, bvals, TS=TS, typeTS, TSextra, alpha=alpha, B=B)
    if(length(list(...))==0) rownames(pwr)=NULL
    if(length(list(...))==1) rownames(pwr)=avals
    if(length(list(...))==2) rownames(pwr)=paste0(avals,"|",bvals)  
    if(doMethods[1]!="all") pwr=pwr[, doMethods, drop=FALSE]
    return(round(pwr, 4))
  }

# Run power routines for continuous data
  if(!LargeSampleOnly) {
    if(maxProcessor1==1) {
      tmp=powerC(rxy, avals, bvals, TS, typeTS, TSextra, B)
      Data=tmp$Data
      Simulated=tmp$Simulated
      paramalt=tmp$paramalt
    }    
    else {
      cl=parallel::makeCluster(maxProcessor1)
      z=parallel::clusterCall(cl, powerC, 
                              rxy,  avals, bvals,
                              TS, typeTS, TSextra, round(B[1]/maxProcessor1))
      parallel::stopCluster(cl)
      Simulated=z[[1]][["Simulated"]]
      Data=z[[1]][["Data"]]
      paramalt=z[[1]][["paramalt"]]
      for(i in 2:maxProcessor1) {
        Simulated=rbind(Simulated, z[[i]][["Simulated"]])
        Data=rbind(Data, z[[i]][["Data"]])
        paramalt=rbind(paramalt,z[[i]][["paramalt"]])
      }  
    }
    pwr=matrix(0, length(avals), length(TS_data))
    colnames(pwr)=names(TS_data)
    for(i in seq_along(avals)) {
      Index=c(1:nrow(Data))[paramalt[,1]==avals[i]&paramalt[,2]==bvals[i]]
      tmpD=Data[Index, , drop=FALSE]
      tmpS=Simulated[Index, , drop=FALSE]
      crtval=apply(tmpS, 2, quantile, prob=1-alpha, na.rm=TRUE)
      for(j in seq_along(crtval)) 
        pwr[i, j]=sum(tmpD[ ,j]>crtval[j])/nrow(tmpD)
    }
  }  
  pwrothers=NULL
  if(missing(rnull) & (typeTS %in% c(1, 4))) {
     if(maxProcessor2==1) {
          pwrothers=power_pvals(rxy, avals, bvals,  
                              TS=TS, typeTS=typeTS, TSextra=TSextra,
                              nbins=nbins, minexpcount=minexpcount, 
                              Ranges=Ranges, alpha=alpha, B=B)
      }  
      else { 
         cl <- parallel::makeCluster(maxProcessor2)
         u = parallel::clusterCall(cl, power_pvals, 
            rxy, avals, bvals,
            TS=TS, typeTS=typeTS, TSextra=TSextra,
            nbins=nbins, minexpcount=minexpcount, Ranges=Ranges,
            alpha=alpha, B=round(B/maxProcessor2))
        parallel::stopCluster(cl)  
        # Average power over cores  
        pwrothers=u[[1]]
        for(i in 2:maxProcessor2) pwrothers=pwrothers+u[[i]]
        pwrothers = pwrothers/maxProcessor2
      }
  }
  if(LargeSampleOnly) pwr=pwrothers
  else if(!CustomTS) pwr = cbind(pwr, pwrothers)
  if(length(list(...))==0) rownames(pwr)=NULL
  if(length(list(...))==1) rownames(pwr)=avals
  if(length(list(...))==2) rownames(pwr)=paste0(avals,"|",bvals)
  if(doMethods[1]!="all") pwr=pwr[, doMethods, drop=FALSE]
  round(pwr, 4)
}

#' Helper function to find test statistics of simulated data.
#' @param  dta a list
#' @param TS test statistic routine
#' @param typeTS type of routine
#' @param TSextra a list
#' @param B number of simulation runs
#' @return a matrix
#' @keywords internal
#' @export
simTS=function(dta, TS, typeTS, TSextra, B) {
  A=matrix(0, B, length(calcTS(dta, TS, typeTS, TSextra)))
  for(i in 1:B) {
    simdta=gen_sim_data(dta, TSextra)
    if(typeTS==1) TSextra$distances=TSextra$dist(simdta)
    A[i,]=calcTS(simdta, TS, typeTS, TSextra)
  }
  A
}

#' Helper function to find test statistics of simulated data.
#' @param  dta a list
#' @param TS test statistic routine
#' @param typeTS type of routine
#' @param TSextra a list
#' @param A a matrix
#' @param Continuous logical
#' @param Ranges a matrix
#' @param nbins a vector
#' @param minexpcount an integer
#' @param B number of simulation runs
#' @return a matrix
#' @keywords internal
#' @export
simpvals=function(dta, TS, typeTS, TSextra, A, Continuous, 
                  Ranges, nbins, minexpcount, B) {
  num_tests=length(calcTS(dta, TS, typeTS, TSextra))
  pvalsTS=matrix(0, B, num_tests)
  Dim=ncol(dta$x)
  if(Continuous) {
       pvalsChi=matrix(0, B, 2)
       colnames(pvalsChi)=c("ES","EP")
       tmp=TS_cont_pval(dta$x, dta$y)$p.values
       pvalsOther=matrix(0, B, length(tmp))
       colnames(pvalsOther)=names(tmp)
  }   
  else {
      pvalsChi=matrix(0, B, 1)
      colnames(pvalsChi)=c("Chisquare")
      pvalsOther=NULL
  }  
  for(i in 1:B) {
    simdta=gen_sim_data(dta, TSextra)
    if(typeTS==1) TSextra$distances=TSextra$dist(simdta)
    tmp=calcTS(simdta, TS, typeTS, TSextra)
    if(Continuous) {
        pvalsOther[i, ]=TS_cont_pval(simdta$x, simdta$y)$p.values 
        if(Dim==2) 
           pvalsChi[i, ]=chisq2D_test_cont(simdta$x, simdta$y, Ranges, nbins, minexpcount)$p.values
    }
    else pvalsChi[i, ]=chisq2D_test_disc(simdta, minexpcount)$p.values
    for(j in 1:num_tests) pvalsTS[i,j]=pvalsTS[i,j]+sum(tmp[j]>A[,j])/nrow(A) 
  }
  colnames(pvalsTS)=names(tmp)
  list(pvalsTS=pvalsTS, pvalsOther=pvalsOther, pvalsChi=pvalsChi)
}

#' Adjusted p values
#' 
#' This function runs a number of two sample tests using Rcpp and parallel computing and then finds the correct p value for the combined tests.
#' 
#' For details consult the vignette("MD2sample","MD2sample")
#' 
#' @param  x  Continuous data: either a matrix of numbers, or a list with two matrices called x and y.
#'                             if it is a matrix Observations are in different rows.
#'            Discrete data: a vector of counts or a matrix with columns named vals_x, vals_y, x and y.
#' @param  y a matrix of numbers if data if data is continuous or a vector of counts  if data is discrete.
#' @param  vals_x =NA, a vector of values for discrete random variable, or NA if data is continuous.
#' @param  vals_y =NA, a vector of values for discrete random variable, or NA if data is continuous.
#' @param  B =c(5000, 1000), number of simulation runs for permutation test and for estimation
#'         of the empirical distribution function.
#' @param  nbins =c(5, 5), number of bins for chi square tests (2D only).
#' @param  minexpcount = 5, minimum required expected counts for chi-square tests.
#' @param  samplingmethod ="Binomial" or "independence" for discrete data.
#' @param  Ranges =matrix(c(-Inf, Inf, -Inf, Inf),2,2) a 2x2 matrix with lower and upper bounds.
#' @param  DoTransform =TRUE, should data be transformed to interval (0,1)?
#' @param  rnull routine for parametric bootstrap.
#' @param  SuppressMessages = FALSE, print informative messages?
#' @param  maxProcessor number of cores for parallel processing.
#' @param  doMethods  Which methods should be included? If missing a small number of methods that generally have good power are used.
#' @return NULL, results are printed out.
#' @examples
#' #Note that the number of simulation runs B is very small to
#' #satisfy CRAN's run time constraints. 
#' #Two continuous data sets from a multivariate normal:
#' x = mvtnorm::rmvnorm(100, c(0,0))
#' y = mvtnorm::rmvnorm(120, c(0,0))
#' twosample_test_adjusted_pvalue(x, y, maxProcessor=1, B=20)
#' #Two discrete data sets from some distribution:
#' x = table(sample(1:4, size=1000, replace = TRUE))
#' y = table(sample(1:4, size=500, replace = TRUE, prob=c(1, 1.5, 1, 1)))
#' twosample_test_adjusted_pvalue(x, y, rep(1:2,2), rep(1:2, each=2), maxProcessor=1, B=20)
#' @export
twosample_test_adjusted_pvalue=function(x, y, vals_x=NA, vals_y=NA,  
                        B=c(5000, 1000), nbins=c(5,5),
                        minexpcount=5, samplingmethod="Binomial",
                        Ranges =matrix(c(-Inf, Inf, -Inf, Inf),2,2),
                        DoTransform=TRUE, rnull, SuppressMessages=FALSE, 
                        maxProcessor, doMethods) {
    default.methods = list(cont=c("ES", "CvM", "AZ", "NN5", "BG"), 
                           disc=c("Chisquare", "KS", "AZ", "CvM"))
    all.methods = list(cont=c("KS", "K", "CvM","AD","NN1", "NN5", "AZ","BF",
                              "BG", "FR", "NN0", "CF1", "CF2", "CF3", "CF4",
                              "ES", "EP"),
                       disc=c("KS", "K", "CvM","AD","NN","AZ","BF","Chisquare"))                                          
    if(length(B)==1) B=c(B, B)
    if(!is.numeric(samplingmethod))
      samplingmethod=ifelse(samplingmethod=="independence", 1, 2)
    if(missing(y)) {
      if(is.list(x)) {#Continuous Data
        if(!SuppressMessages) message("Data is assumed to be continuous")
        Continuous=TRUE
        dta=x
        y=x$y
        x=x$x 
        Dim=ncol(x)
      }
      else {#Discrete data
        if(!SuppressMessages) message("Data is assumed to be discrete")
        Continuous=FALSE
        Dim=2
        dta=x
        x=matrix(1:4,2,2)#just some dummy numbers
        y=matrix(1:4,2,2)
        dta=list(x=dta[,"x"], y=dta[,"y"], vals_x=dta[,"vals_x"], vals_y=dta[,"vals_y"])
        DoTransform=FALSE
      }
    }
    else {
      Continuous=ifelse(any(is.na(c(vals_x, vals_y))), TRUE, FALSE)
      if(Continuous) {
        if(!SuppressMessages) message("Data is assumed to be continuous")
        dta=list(x=x, y=y)
        Dim=ncol(x)
      } 
      else {
        if(!SuppressMessages) message("Data is assumed to be discrete")
        Dim=2
        dta=list(x=x, y=y, vals_x=vals_x, vals_y=vals_y)
        x=matrix(1:4,2,2)#just some dummy numbers
        y=matrix(1:4,2,2)
        DoTransform=FALSE
      }    
    }
    if(Continuous) {
      if(nrow(y)<nrow(x)) { #switch x and y so that nx>ny
        tmp=y
        y=x
        x=tmp
        dta=list(x=x, y=y)
      }
      rawdta=dta
      if(DoTransform) {
          dta=transform01(dta)
          x=dta$x
          y=dta$y
          Ranges=matrix(c(0, 1, 0, 1),2,2)
      }
    }
    if(Continuous) 
         TSextra = list(knn=function(x) FNN::get.knn(x, 5)$nn.index,
                dist=function(dta) find_dist(dta),
                distances=find_dist(dta),
                DoTransform=DoTransform)
    else TSextra = list(
                dist=function(dta) NULL,
                organize=function(dta) dta=dta[order(dta[,1], dta[,2]), ],
                samplingmethod=samplingmethod)
    if(!missing(rnull)) {
      if(Continuous) 
        TSextra=c(TSextra, rnull=rnull, rawdta=list(rawdta))
      else TSextra=c(TSextra, rnull=rnull)
    } 
    outchi=list(statistics=NULL, p.value=NULL)
    outpvals=list(statistics=NULL, p.value=NULL)
    # what methods are to be run?  
    if(Continuous) {
        if(Dim==2) {
          if(length(nbins)==1) nbins=c(nbins, nbins)
          outchi = chisq2D_test_cont(x, y, Ranges, nbins, minexpcount)
        }  
        outpvals=TS_cont_pval(x, y) #Methods that find p values
        typeTS=1
        TS=TS_cont
        dta=list(x=x, y=y)
    }
    else {
        typeTS=4
        outchi = chisq2D_test_disc(dta,  minexpcount)
        TS=TS_disc
    }    
    TS_data=calcTS(dta, TS, typeTS, TSextra)
    if(any(is.null(names(TS_data)))) {
      if(!SuppressMessages) message("output of TS routine has to be a named vector!")
      return(NULL)
    }  
#   Find matrix of values of test statistics for simulated data
    if(missing(maxProcessor)) {
       maxProcessor=parallel::detectCores(logical = FALSE)-1
       if(!SuppressMessages) message(paste("Using ", maxProcessor," cores.."))
    }   
    if(maxProcessor==1) A=simTS(dta, TS, typeTS, TSextra, B[1])
    else {
       cl=parallel::makeCluster(maxProcessor)
       z=parallel::clusterCall(cl, simTS, dta, TS, typeTS, 
                               TSextra, round(B[1]/maxProcessor))
       A=z[[1]]
       for(i in 2:maxProcessor) A=rbind(A, z[[i]])
       B[1]=nrow(A)

    }
    num_tests=ncol(A)
    tmp=TS_data
    pvalsdta=rep(0, num_tests)
    for(j in 1:num_tests) pvalsdta[j]=pvalsdta[j]+sum(tmp[j]<A[,j])/nrow(A)    
    if(Continuous) {
        pvalsdta=c(pvalsdta, TS_cont_pval(x, y)$p.values) 
        if(length(nbins)==1) nbins=c(nbins, nbins)
        if(Dim==2) chitmp=chisq2D_test_cont(x, y, Ranges, nbins, minexpcount)$p.values
        else chitmp=rep(0, 2)
        pvalsdta=c(pvalsdta, chitmp)
        names(pvalsdta)=all.methods$cont
    }   
    else {
        pvalsdta=c(pvalsdta, chisq2D_test_disc(dta, minexpcount)$p.values)
        names(pvalsdta)=all.methods$disc
    }
    if(maxProcessor==1) {
        tmp=simpvals(dta, TS, typeTS, TSextra, A, Continuous, 
                 Ranges, nbins, minexpcount, B[2])
        pvalsTS=tmp$pvalsTS
        pvalsOther=tmp$pvalsOther
        pvalsChi=tmp$pvalsChi
    }
    else {
      z=parallel::clusterCall(cl, simpvals, dta, TS, 
                  typeTS, TSextra, A, Continuous, 
                  Ranges, nbins, minexpcount, B[2]/maxProcessor)
      pvalsTS=z[[1]][[1]]
      pvalsOther=z[[1]][[2]]
      pvalsChi=z[[1]][[3]]
      for(i in 2:maxProcessor) {
        pvalsTS=rbind(pvalsTS, z[[i]][[1]])
        pvalsOther=rbind(pvalsOther, z[[i]][[2]])
        pvalsChi=rbind(pvalsChi, z[[i]][[3]])
      }  
      B[1]=nrow(pvalsTS)
      parallel::stopCluster(cl)
    }
    if(missing(doMethods)) {
      if(Continuous) doMethods=default.methods[["cont"]]
      else doMethods=default.methods[["disc"]]
    }
    if(doMethods[1]=="all"){
      if(Continuous) doMethods=all.methods[["cont"]]
      else doMethods=all.methods[["disc"]]
    }
    if(Continuous & Dim>2) {
      doMethods=doMethods[doMethods!="ES"]
      doMethods=doMethods[doMethods!="EP"]
    }
    pvals=cbind(pvalsTS, pvalsOther, pvalsChi) 
    pvals=pvals[ ,doMethods,drop=FALSE]
    pvalsdta=pvalsdta[doMethods]
    minp_x=min(pvalsdta)
    minp_sim=apply(pvals[, ,drop=FALSE], 1, min)
    z=seq(0, 1, length=250)
    y=z
    for(i in 1:250) y[i]=sum(minp_sim<=z[i])/length(minp_sim)
    I=c(1:250)[z>minp_x][1]-1
    slope=(y[I+1]-y[I])/(z[I+1]-z[I])
    minp_adj=round(y[I]+slope*(minp_x-z[I]),4)
    message("p values of individual tests:")
    for(i in seq_along(pvalsdta)) 
        message(paste(names(pvalsdta)[i],": ", round(pvalsdta[i],4)))
    message(paste0("adjusted p value of combined tests: ", minp_adj))
    
}

#' Change Marginals of 2D Data
#' 
#' This function modifies the marginals of some data set
#' 
#' @param dta a list with two data sets
#' @param which method for modifying the data sets.
#' @param theta how much to modify
#' @keywords internal
#' @return a list of functions
#' @export
change.marginals=function(dta, which, theta) {
  if(which=="Exponential") {
    for(i in 2:ncol(dta$x)) {
      dta$x[, i] =  qexp(dta$x[, i], 1)
      dta$x[, i][dta$x[, i]>3]=3/dta$x[, i][dta$x[, i]>3]
      dta$y[, i] =  qexp(dta$y[, i], theta)
      dta$y[, i][dta$y[, i]>3]=3/dta$y[, i][dta$y[, i]>3]
    }
  }
  if(which=="Linear") {
    if(theta>0)
       for(i in 2:ncol(dta$y)) 
         dta$y[,i]=(theta-1+sqrt((1-theta)^2+4*theta*dta$y[,i]))/2/theta
  }
  if(which=="Normal") {
    for(i in 2:ncol(dta$x)) {
      dta$x[,i] =  qnorm(dta$x[, i])
      dta$y[,i] =  qnorm(dta$y[, i], 0, theta)
    }  
  }
  dta
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom stats qexp
#' @importFrom stats qnorm
#' @importFrom stats rbeta
#' @importFrom stats rexp
#' @importFrom stats runif
## usethis namespace: end
NULL

#' This function does some rounding to nice numbers
#' @param  x a list of two vectors
#' @param  d =4 number of digits to round to
#' @return A list with rounded vectors
signif.digits=function(x, d=4) {
   stat=x$statistics
   pval=x$p.values
   pval=round(pval, 4)
   y=strsplit(as.character(stat), "\\.")
   z=rep(0, length(y))
   for(i in 1:length(y)) {
      m=nchar(y[[i]][1])
      if(m>d) z[i]=round(stat[i], d-m)
      if(m<d & floor(stat[i])!=0) z[i]=round(stat[i], d-m+1)
      if(floor(stat[i])==0) {
        k=0
        repeat {
          k=k+1
          if(abs(stat[i]<1e-6)) break
          if(k>nchar(y[[i]][2])) break
          if(as.numeric(substr(y[[i]][2], 1, k))!=0) break
        }
        if(abs(stat[i]<1e-10)) tmp="0"
        else tmp=paste0("0.", paste0(rep("0",k-1),collapse=""), substr(y[[i]][2],k,d+k-1), collapse = "")
        z[i]=as.numeric(tmp)   
      } 
   }
   names(z)=names(pval)
   list(statistics=z, p.values=pval)
}

#' Benchmarking for Multivariate Two-Sample Tests
#' 
#' This function runs the case studies included in the package.
#' 
#' For details consult vignette(package="MD2sample")
#' 
#' @param Continuous =TRUE, run cases for continuous data.
#' @param study either the name of the study, or its number in the list. If missing all the studies are run.
#' @param TS routine to calculate new test statistics. 
#' @param TSextra list passed to TS (optional).
#' @param With.p.value =FALSE, does user supplied routine return p values?
#' @param nsample = 200, desired sample size. 200 is used in included case studies.
#' @param alpha =0.05,  type I error probability of tests. 0.05 is used in included case studies.
#' @param param_alt (list of) values of parameter under the alternative hypothesis. 
#'                  If missing included values are used.
#' @param SuppressMessages =FALSE, should informative messages be printed?
#' @param B = 1000, number of simulation runs.
#' @param  maxProcessor  number of cores to use. If missing the number of physical cores-1 
#'             is used. If set to 1 no parallel processing is done.
#' @return A (list of ) matrices of p.values.
#' @examples
#' #The new test is a (included) chi square test:
#' TSextra=list(which="pval", nbins=rbind(c(3,3), c(4,4)))
#' run.studies(Continuous=TRUE, study=c("NormalD2", "tD2"), 
#'           TS=MD2sample::chiTS.cont, TSextra=TSextra, 
#'           With.p.value=TRUE, B=100)
#' @export
run.studies <- function(Continuous=TRUE, study, TS, TSextra, With.p.value=FALSE,  
          nsample=200, alpha=0.05, param_alt, 
          SuppressMessages =FALSE, B=1000, maxProcessor) {
  list.of.studies=MD2sample::case.studies(ReturnCaseNames=TRUE)
  if(missing(study)) study=1:length(list.of.studies)
  if(is.numeric(study)) study=list.of.studies[study]
  if(!Continuous) study=study[!endsWith(study,"D5")]
  RerunIncludedTests=FALSE
  if(missing(TS)) {
      RerunIncludedTests=TRUE
      if(!missing(param_alt)) {
          if(!SuppressMessages) 
            message("if TS is missing included tests are rerun for new parameters")
      }
  }    
  if(RerunIncludedTests) {
      if(length(study)>1) {
        if(is.matrix(param_alt)) {
           if(nrow(param_alt)!=length(study))
             if(!SuppressMessages) 
               message("param_alt should be a matrix with the one row for each study")
          
        }
        else {
          if(!SuppressMessages) message("if more than one study is run 
      param_alt has to be a matrix with the one row for each study")
          return(NULL)
        }  
      }
      else param_alt=rbind(param_alt)
      out=as.list(1:length(study))
      names(out)=study
      for(i in seq_along(study)) {
        tmp=MD2sample::case.studies(study[i])
        if(!Continuous) 
           tmp=MD2sample::case.studies(study[i], nbins=tmp$nbins)
        if(!SuppressMessages) message(paste("Running case study", study[i],"..."))
        out[[i]]=MD2sample::twosample_power(tmp$f, param_alt[i,], 
                      alpha=alpha,  SuppressMessages=SuppressMessages, 
                      B=B, maxProcessor=maxProcessor) 
      }  
      if(length(out)==1) return(out[[1]])
      return(out)
  } 
  if(Continuous) typeTS=length(formals(TS))
  else typeTS=ifelse(length(formals(TS))==3, 6, 5)  
  if(Continuous) pwr=MD2sample::power_studies_results[["Alt Cont"]]
  else pwr=MD2sample::power_studies_results[["Alt Disc"]]
  nstudy=length(study)
  for(i in seq_along(study)) {
    tmp=case.studies(study[i], nsample)
    if(!Continuous) 
      tmp=case.studies(study[i], nbins=tmp$nbins)
    if(!SuppressMessages) message(paste("Running case study", study[i],
                                       "with param_alt=", tmp$param_alt[2]))
    rxy=function(a,b=0) tmp$f(a)
    if(With.p.value) {
        if(missing(TSextra)) TSextra=list(aaa=0)
        pwrTS=MD2sample::power_pvals(rxy, tmp$param_alt[2], 
            0, TS, typeTS, TSextra, alpha=alpha, B=B)
    }    
    else  pwrTS=MD2sample::twosample_power(tmp$f, tmp$param_alt[2], 
                                   TS=TS, TSextra=TSextra, alpha=alpha,  
                                   SuppressMessages=SuppressMessages,
                                   B=B, maxProcessor=maxProcessor)   
    if(i==1) {
      allpwr=pwr[study, , drop=FALSE]
      for(m in 1:length(pwrTS)) allpwr=cbind(0, allpwr)
    }  
    allpwr[study[i], 1:length(pwrTS)]=pwrTS
  }
  colnames(allpwr) = c(colnames(pwrTS), colnames(pwr))
  if(nstudy>1) {
    a1=apply(allpwr, 1, rank)
    names(a1)=c(colnames(pwrTS), colnames(pwr))
    message("Average number of times a test is close to best:")
    print(sort(apply(a1,1,mean)))
  }  
  allpwr
}

#' This function runs a number of two sample tests which find their own p value.
#' @param  x  a matrix of numbers if data is continuous or of counts  if data is discrete.
#' @param  y a matrix of numbers if data is continuous or of counts  if data is discrete.
#' @return A list of two numeric vectors, the test statistics and the p values. 
#' @export 
TS_cont_pval = function(x, y) {
   D=ncol(x)
   nx=nrow(x)
   ny=nrow(y)
   n=nx+ny
   # test by 
   out=list(1:7, 1:7)
   names(out)=c("statistics", "p.values")
   names(out[[1]])=c("FR", "NN0", paste0("CF",1:4), "Ball")
   names(out[[2]])=c("FR", "NN0", paste0("CF",1:4), "Ball")   
   
   # test by Friedman and Rafski
   tmp=try(FR.test(x,y),TRUE)
   if(!is.list(tmp)) tmp=list(statistic=-99, p.value=-99)
   out[[1]][1]=tmp[[1]]
   out[[2]][1]=tmp[[2]] 

#  Nearest Neighbor variant   
   NN = c(FNN::get.knn(rbind(x,y), 1)$nn.index)
   out[[1]][2]=sum(NN[1:nx]<=nx)/n
   out[[2]][2]=1-stats::pbinom(n*out[[1]][2], nx, nx/n)
#  four tests by Chen and Friedman
   tmp=try(edge.tests(x, y), TRUE)
   if(!is.list(tmp)) tmp=list(statistic=-99, p.value=-99)
   out[[1]][3:6]=tmp[[1]]
   out[[2]][3:6]=tmp[[2]]
   tmp=try(Ball::bd.test(x, y, method="limit"), TRUE)
   if(!is.list(tmp)) tmp=list(statistic=-99, p.value=-99)
   out[[1]][7]=tmp[["statistic"]]
   out[[2]][7]=tmp[["p.value"]]
#  return p values   

   out
}

#' This function finds the power for tests that find their own p values
#' @param  rxy  a function to generate data
#' @param  avals vector of parameter values
#' @param  bvals vector of parameter values
#' @param  TS routine that runs the test and returns p values
#' @param  typeTS  type of problem, continuous or discrete
#' @param  TSextra =list(a=0) a list of things passed to TS, if needed
#' @param  nbins =c(5, 5), bins for chi square tests.
#' @param  minexpcount =5, lowest required count for chi-square test
#' @param  Ranges =matrix(c(-Inf, Inf, -Inf, Inf),2,2) a 2x2 matrix with lower and upper bounds
#' @param  alpha =0.05 type I error probability of test
#' @param  B =1000 number of simulation runs
#' @keywords internal
#' @return A matrix of power values
#' @export
power_pvals = function(rxy, avals, bvals, TS, typeTS, TSextra=list(a=0),
                       nbins=c(5,5), minexpcount=5, 
                       Ranges =matrix(c(-Inf, Inf, -Inf, Inf),2,2),
                       alpha=0.05, B=1000) {
  
   dta = rxy(avals[1], bvals[1])
   Continuous=TRUE
   if(is.matrix(dta)) Continuous=FALSE
   DoTransform=FALSE
   if(Continuous) {
     if("DoTransform" %in% names(TSextra)) DoTransform=TSextra$DoTransform
     else {
       z=c(dta$x, dta$y)
       if(min(z)<0 | max(z)>1) DoTransform=TRUE
     }
     if(DoTransform) {
       dta = transform01(dta)
       Ranges =matrix(c(0, 1, 0, 1),2,2)
     }   
     Dim = ncol(dta$x)
     DoChi=FALSE
     if(typeTS==1 & Dim==2) DoChi=TRUE
     if(typeTS==1) {
       typeTS=2 
       TS=function(x,y) TS_cont_pval(x, y)$p.values
     }   
     TS_data = calcTS(dta, TS, typeTS, TSextra)
     numtests=length(TS_data)
     if(typeTS<=3) Continuous = TRUE
     else Continuous = FALSE
     pwr=matrix(0, length(avals), numtests+ifelse(DoChi, 2, 0))
     if(DoChi) colnames(pwr) = c(names(TS_data), "ES", "EP")
     else colnames(pwr) = names(TS_data)
     rownames(pwr)=avals
     for(i in 1:length(avals)) {
       NoGood=NULL
       for(j in 1:B) {
         dta=rxy(avals[i], bvals[i])
         if(DoTransform) dta=transform01(dta)
         TS_sim = calcTS(dta, TS, typeTS, TSextra)
         if(any(is.nan(TS_sim))) {j=j-1;next}
         if(any(TS_sim<0)) {
             NoGood=c(NoGood, seq_along(TS_sim)[TS_sim<0])
         }
         pwr[i, 1:numtests] = pwr[i, 1:numtests] + ifelse(TS_sim<alpha, 1, 0)
         if(DoChi) {
           if(length(nbins)==1) nbins=c(nbins, nbins)
           chi=chisq2D_test_cont(dta$x, dta$y, Ranges, nbins, minexpcount,
                                 SuppressMessages=TRUE)$p.values
           pwr[i, numtests+1:2] = pwr[i, numtests+1:2] + ifelse(chi<alpha, 1, 0)
         }   
       }
       pwr[i, ] = pwr[i, ,drop=FALSE]/B
       if(length(NoGood)>0) pwr[i, NoGood]=NA
     }
   }   
   else {#Discrete Data
     if(typeTS==4) {      
        pwr=matrix(0,  length(avals), 1)
        colnames(pwr)="Chisquare"
        rownames(pwr)=avals
        for(i in 1:length(avals)) {
          for(j in 1:B) {
            dta=rxy(avals[i], bvals[i])
            chi=chisq2D_test_disc(dta, minexpcount)$p.values
            pwr[i,1] = pwr[i,1] + ifelse(chi<alpha, 1, 0)
          }
          pwr[i,1] = pwr[i,1]/B          
        }
     }
     else {      
       dta=rxy(avals[1], bvals[1])
       if(typeTS>=4) {
          dn=colnames(dta)
          if("x"%in%dn & "y"%in%dn & "vals_x"%in%dn & "vals_y"%in%dn)
             dta=list(x=dta[,"x"],y=dta[,"y"],vals_x=dta[,"vals_x"],vals_y=dta[,"vals_y"])
          else {
            dta=list(x=dta[,3],y=dta[,4],vals_x=dta[,1],vals_y=dta[,2])
            message("data matrix is missing column names. Assumed order is:")
            message("vals_x, vals_y, x, y")
          }
       }
       TS_sim=calcTS(dta, TS, typeTS, TSextra)
       pwr=matrix(0,  length(avals), length(TS_sim))
       colnames(pwr)=names(TS_sim)
       rownames(pwr)=avals
       for(i in 1:length(avals)) {
         for(j in 1:B) {
           dta=rxy(avals[i], bvals[i])
           if(typeTS>=4) {
             if("x"%in%dn & "y"%in%dn & "vals_x"%in%dn & "vals_y"%in%dn)
               dta=list(x=dta[,"x"],y=dta[,"y"],vals_x=dta[,"vals_x"],vals_y=dta[,"vals_y"])
             else {
               dta=list(x=dta[,3],y=dta[,4],vals_x=dta[,1],vals_y=dta[,2])
             }
           }
           TS_sim=calcTS(dta, TS, typeTS, TSextra)
           for(k in 1:ncol(pwr)) 
              pwr[i,k] = pwr[i,k] + ifelse(TS_sim[k]<alpha, 1, 0)/B
         }
       }
     }
   }
   pwr
}

#' This function does the chi square two sample test for continuous data in two dimensions
#' @param dta_x  a matrix of numbers 
#' @param dta_y a matrix of numbers 
#' @param Ranges =matrix(c(-Inf, Inf, -Inf, Inf),2,2) a 2x2 matrix with lower and upper bounds
#' @param nbins =c(5,5) number of bins in x and y direction
#' @param minexpcount =5 minimum counts required per bin
#' @param SuppressMessages =FALSE, print informative messages?
#' @return a list with statistics and p values
chisq2D_test_cont=function(dta_x, dta_y, Ranges=matrix(c(-Inf, Inf, -Inf, Inf),2,2),
                   nbins=c(5, 5), minexpcount=5, SuppressMessages=FALSE) {
  if(ncol(dta_x)!=2) {message("Test is for two dimensional data only!");return(NULL)}
  dta_xy=rbind(dta_x, dta_y)
  low=low=apply(dta_xy,2,min)
  if(is.finite(Ranges[1,1])) low[1]=Ranges[1,1]
  if(is.finite(Ranges[1,2])) low[2]=Ranges[1,2]
  high=apply(dta_xy,2,max)
  if(is.finite(Ranges[2,1])) high[1]=Ranges[2,1]
  if(is.finite(Ranges[2,2])) high[2]=Ranges[2,2]
  tmp=c(0,0)
  names(tmp)=c("ES", "EP")
  out=list(statistics=tmp, p.values=tmp, df=tmp)
  for(l in 1:2) {
    if(l==1) {
      grd=as.list(1:2)
      for(i in 1:2) { # equal space grid
        grd[[i]]=seq(low[i], high[i], length=nbins[i]+1)
      }      
    }
    else {
      grd=as.list(1:2)
      for(i in 1:2) { #equal probability grid
        grd[[i]]=quantile(dta_xy[,i], 0:nbins[i]/nbins[i])
        grd[[i]][c(1,nbins[i]+1)]=c(low[i], high[i])
      }
    } 
    A_x=0*dta_x
    A_y=0*dta_y
    A_xy=0*dta_xy
    for(i in 1:2) {
       A_x[,i]=cut(dta_x[,i], grd[[i]], labels=1:nbins[i],include.lowest =TRUE)
       A_y[,i]=cut(dta_y[,i], grd[[i]], labels=1:nbins[i],include.lowest =TRUE)
       A_xy[,i]=cut(dta_xy[,i], grd[[i]], labels=1:nbins[i],include.lowest =TRUE)
    }
    O_xy=matrix(0, nbins[1], nbins[2])
    rownames(O_xy)=1:nbins[1]
    colnames(O_xy)=1:nbins[2]
    tmp=table(A_xy[,1],A_xy[,2])
    O_xy[rownames(tmp),colnames(tmp)]=tmp
    O_x=0*O_xy
    tmp=table(A_x[,1],A_x[,2])
    O_x[rownames(tmp),colnames(tmp)]=tmp
    O_y=0*O_xy
    tmp=table(A_y[,1],A_y[,2])
    O_y[rownames(tmp),colnames(tmp)]=tmp
    step=1
    repeat {
       if(min(O_xy, na.rm=TRUE)>=minexpcount) break
       lowest = c(1, 1)
       tmp = max(O_xy, na.rm = TRUE)
       for(i in 1:nbins[1]) {
         for(j in 1:nbins[2]) {
           if(!is.na(O_xy[i,j]) && O_xy[i,j]<tmp) {
             tmp=O_xy[i,j]
             lowest=c(i,j)
           }
         }
       }
       I=lowest[1]+c(-1,0,1)*step
       I=I[1<=I&I<=nbins[1]]
       J=lowest[2]+c(-1,0,1)*step
       J=J[1<=J&J<=nbins[2]]
       tmp=max(O_xy, na.rm = TRUE)
       nearest=c(-1,-1)
       for(i in I)
         for(j in J) {
           if(i==lowest[1]&&j==lowest[2]) next
           if(!is.na(O_xy[i,j])&&O_xy[i,j]<tmp) {
             tmp=O_xy[i,j]
             nearest = c(i,j)
           }
         }
       if(step>min(nbins)) {
         if(!SuppressMessages) {
           message("Not enough data for the number of bins.")
           message(paste0("Will try nbins=(",nbins[1]-1,",",nbins[2]-1,")"))
         }   
         return(chisq2D_test_cont(dta_x, dta_y, Ranges=Ranges, 
                        nbins=nbins-1, minexpcount=minexpcount))
       }   
       if(nearest[1]<0) {step=step+1;next}
       
       O_x[lowest[1], lowest[2]] = O_x[lowest[1], lowest[2]] + 
                                   O_x[nearest[1], nearest[2]]
       O_x[nearest[1], nearest[2]] = NA
       O_y[lowest[1], lowest[2]] = O_y[lowest[1], lowest[2]] + 
         O_y[nearest[1], nearest[2]]
       O_y[nearest[1], nearest[2]] = NA
       O_xy[lowest[1], lowest[2]] = O_xy[lowest[1], lowest[2]] + 
         O_xy[nearest[1], nearest[2]]
       O_xy[nearest[1], nearest[2]] = NA
       step=1
    }
    O_x=O_x[!is.na(O_x)]
    O_y=O_y[!is.na(O_y)]
    O_xy=O_xy[!is.na(O_xy)]
    s=sqrt(sum(O_x)/sum(O_y))
    chi=sum((O_x/s-s*O_y)^2/O_xy)
    df=length(O_x)-1
    out[[1]][l]=chi
    out[[2]][l]=1-stats::pchisq(chi,df)
    out[[3]][l]=df
  }  
  out
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' sort vector y by values in vector x
#' 
#' @param y numeric vector
#' @param x numeric vector
#' @keywords internal
#' @return numeric vector
xysort <- function(y, x) {
    .Call(`_MD2sample_xysort`, y, x)
}

#' equivalent to R command order
#' 
#' @param x numeric vector
#' @keywords internal
#' @return integer vector
Order <- function(x) {
    .Call(`_MD2sample_Order`, x)
}

#' equivalent to R command rank
#' 
#' @param x numeric vector
#' @keywords internal
#' @return integer vector
Rank <- function(x) {
    .Call(`_MD2sample_Rank`, x)
}

#' Find test statistics for continuous data
#' 
#' @param x A matrix.
#' @param y A matrix
#' @param TSextra list with some helper functions
#' @keywords internal
#' @return A numeric vector with test statistics
TS_cont <- function(x, y, TSextra) {
    .Call(`_MD2sample_TS_cont`, x, y, TSextra)
}

#' Find test statistics for discrete data
#' @param x a vector of integers.
#' @param y a vector of integers.
#' @param vx a vector of real.
#' @param vy a vector of real.
#' @param TSextra list with extra infos
#' @keywords internal
#' @return A numeric vector with test statistics
TS_disc <- function(x, y, vx, vy, TSextra) {
    .Call(`_MD2sample_TS_disc`, x, y, vx, vy, TSextra)
}

#' This function calculates the test statistics
#' @param  dta data set
#' @param  TS routine
#' @param  typeTS format of TS
#' @param  TSextra list passed to TS function
#' @keywords internal
#' @return A vector of numbers
calcTS <- function(dta, TS, typeTS, TSextra) {
    .Call(`_MD2sample_calcTS`, dta, TS, typeTS, TSextra)
}

#' find pairwise Euclidean distances
#' 
#' @param dta A list of matrices
#' @keywords internal
#' @return A  matrix
find_dist <- function(dta) {
    .Call(`_MD2sample_find_dist`, dta)
}

#' simulate new data
#' @param dta data set
#' @param TSextra extra stuff
#' @keywords internal
#' @return A list of permuted vectors
gen_sim_data <- function(dta, TSextra) {
    .Call(`_MD2sample_gen_sim_data`, dta, TSextra)
}

getI <- function(p) {
    .Call(`_MD2sample_getI`, p)
}

#' simulate continuous data
#' @param x first data set
#' @param y second data set
#' @param TSextra extra stuff
#' @keywords internal
#' @return A list of permuted vectors
gen_cont <- function(x, y, TSextra) {
    .Call(`_MD2sample_gen_cont`, x, y, TSextra)
}

#' simulate new discrete data
#' @param x first data set, counts
#' @param y second data set, counts
#' @param vals_x values of discrete random variable x
#' @param vals_y values of discrete random variable y
#' @param TSextra extra stuff
#' @keywords internal
#' @return A list of permuted vectors
gen_disc <- function(x, y, vals_x, vals_y, TSextra) {
    .Call(`_MD2sample_gen_disc`, x, y, vals_x, vals_y, TSextra)
}

#' Finds the empirical distribution function
#' @param dta a matrix of data points
#' @param pts a matrix of evaluation points
#' @keywords internal
#' @return a numeric vector
mdecdf <- function(dta, pts) {
    .Call(`_MD2sample_mdecdf`, dta, pts)
}

#' Find test statistic for MMD test
#' 
#' @param x A matrix 
#' @param y A matrix 
#' @keywords internal
#' @return A numeric vector with test statistics
mmdagg <- function(x, y) {
    .Call(`_MD2sample_mmdagg`, x, y)
}

#' Find the power of various continuous tests via simutation or permutation.
#' 
#' @param rxy a function that generates x and y data.
#' @param xparam  arguments for r1.
#' @param yparam  arguments for r2.
#' @param TS routine to calculate test statistics for non-chi-square tests
#' @param typeTS indicator for type of test statistics
#' @param TSextra additional info passed to TS, if necessary
#' @param B =1000 number of simulation runs
#' @keywords internal
#' @return A list values of test statistics
powerC <- function(rxy, xparam, yparam, TS, typeTS, TSextra, B = 1000L) {
    .Call(`_MD2sample_powerC`, rxy, xparam, yparam, TS, typeTS, TSextra, B)
}

#' run tests using either simulation or permutation.
#' 
#' @param dta a list with the data
#' @param TS routine to calculate test statistics for non-chi-square tests 
#' @param typeTS type of a test statistic
#' @param TSextra additional info passed to TS, if necessary
#' @param B =5000, number of simulation runs.
#' @keywords internal
#' @return A list with test statistics and p values
testC <- function(dta, TS, typeTS, TSextra, B = 5000L) {
    .Call(`_MD2sample_testC`, dta, TS, typeTS, TSextra, B)
}

#' transform data to interval (0,1).
#' 
#' @param dta a list.
#' @keywords internal
#' @return a list 
transform01 <- function(dta) {
    .Call(`_MD2sample_transform01`, dta)
}

#' This function checks whether the correct methods have been requested
#' @param  doMethods ="all" Which methods should be included?
#' @param  Continuous is data continuous
#' @param  ReturnMethodNames = FALSE should names of methods be returned?
#' @keywords internal
#' @return TRUE/FALSE or a character vector
#' @export 
test_methods=function(doMethods, Continuous, ReturnMethodNames=FALSE) {
    if(doMethods[1]=="all") return(FALSE)
    if(Continuous) methods=
                 c("KS","K","CvM","AD","NN1", "NN5", "AZ","BF","BG",
                   "FR","NN0","CF1","CF2","CF3","CF4",
                   "Ball",  "ES", "EP")
    else methods=c("KS","K","CvM","AD","NN","AZ", "BF","ChiSquare")
    if(ReturnMethodNames) return(methods)
    Good=TRUE
    for(i in seq_along(doMethods)) {
      if(!(doMethods[i]%in%methods)) {Good=FALSE;break}
    }
    if(Good) return(FALSE)
    message(paste0(doMethods[i]," is not an included method for ", 
                   ifelse(Continuous, "continuous", "discrete"), " data!"))
    if(Continuous) {
         message("For continuous data included methods are")
         message("Method               Code")
         message("Kolmogorov-Smirnov   KS")
         message("Kuiper               K")
         message("Cramer-vonMises      CvM")
         message("Anderson-Darling     AD")
         message("1-nearest neighbor   NN1")
         message("5-nearest neighbor   NN5")
         message("Aslan-Zech           AZ")
         message("Baringhaus-Franz     BF")
         message("Biswas-Ghosh         BG")
         message("Friedman-Rafski      FR")
         message("x nearest neighbor   NN0")
         message("Chen-Friedman        CF1-CF4")
         message("Ball Divergence      Ball")
         message("Chi square tests     ES, EP")
    }
    if(!Continuous) {
      message("For discrete data included methods are")
      message("Method               Code")
      message("Kolmogorov-Smirnov   KS")
      message("Kuiper               K")
      message("Cramer-vonMises      CvM")
      message("Anderson-Darling     AD")
      message("Nearest Neigbor      NN")
      message("Aslan-Zech           AZ")
      message("Baringhaus-Franz     BF")
      message("Chi square test      Chisquare")
    }
    TRUE
}

#' Multivariate Two-Sample Tests
#' 
#' This function runs a number of two sample tests using Rcpp and parallel computing.
#' 
#' For details consult vignette("MD2sample","MD2sample")
#' 
#' @param  x  Continuous data: either a matrix of numbers, or a list with two matrices called x and y.
#'                             if it is a matrix Observations are in different rows.
#'            Discrete data: a vector of counts or a matrix with columns named vals_x, vals_y, x and y.
#' @param  y a matrix of numbers if data is continuous or a vector of counts  if data is discrete. 
#' @param  vals_x =NA, a vector of values for discrete random variables, or NA if data is continuous.
#' @param  vals_y =NA, a vector of values for discrete random variables, or NA if data is continuous.
#' @param  TS user supplied routine to calculate test statistics for new tests.
#' @param  TSextra (optional) additional info passed to TS, if necessary.
#' @param  B =5000, number of simulation runs for permutation test.
#' @param  nbins =c(5,5), for chi square tests (2D only).
#' @param  minexpcount =5, lowest required count for chi-square test (2D only).
#' @param  Ranges =matrix(c(-Inf, Inf, -Inf, Inf),2,2), a 2x2 matrix with lower and upper bounds (2D only).
#' @param  DoTransform =TRUE, should data be transformed to unit hypercube?
#' @param  samplingmethod ="Binomial" for Binomial sampling or "independence" for independence sampling.
#' @param  rnull function to generate new data sets for simulation as an alternative to the permutation method.
#' @param  SuppressMessages =FALSE, should informative messages be printed?
#' @param  LargeSampleOnly =FALSE, should only methods with large sample theories be run?
#' @param  maxProcessor  number of cores to use. If missing the number of physical cores-1 
#'             is used. If set to 1 no parallel processing is done.
#' @param  doMethods ="all", Which methods should be included?
#' @return A list of two numeric vectors, the test statistics and the p values. 
#' @examples
#' #Two continuous data sets from a multivariate normal:
#' x = mvtnorm::rmvnorm(100, c(0,0))
#' y = mvtnorm::rmvnorm(120, c(0,0))
#' twosample_test(x, y, B=100, maxProcessor=1)
#' #Using a new test, this one is an (included) chi square test. 
#' #Also enter data as a list:
#' TSextra=list(which="statistics", nbins=rbind(c(3,3), c(4,4)))
#' dta=list(x=x, y=y)
#' twosample_test(dta, TS=chiTS.cont, TSextra=TSextra, B=100, maxProcessor=1)
#' #Two discrete data sets from some distribution:
#' x = table(sample(1:4, size=1000, replace = TRUE))
#' y = table(sample(1:4, size=1000, replace = TRUE, prob=c(1,2,1,1)))
#' vals_x=rep(1:2,2)
#' vals_y=rep(1:2, each=2)
#' twosample_test(x, y, vals_x, vals_y, B=100, maxProcessor=1)
#' #Run a discrete chi square test and enter the data as a matrix:
#' TSextra=list(which="statistics")
#' dta=cbind(x=x, y=y, vals_x=vals_x, vals_y=vals_y)
#' twosample_test(dta, TS=chiTS.disc, TSextra=TSextra, B=100, maxProcessor=1)
#' @export 
twosample_test=function(x, y, vals_x=NA, vals_y=NA, TS, TSextra, B=5000, 
          nbins=c(5,5),minexpcount =5, Ranges=matrix(c(-Inf, Inf, -Inf, Inf),2,2),
          DoTransform=TRUE, samplingmethod="Binomial", rnull,
          SuppressMessages=FALSE, LargeSampleOnly=FALSE, 
          maxProcessor, doMethods="all") {
    if(!is.numeric(samplingmethod))
      samplingmethod=ifelse(samplingmethod=="independence", 1, 2)
    if(missing(y)) {
       if(is.list(x)) {#Continuous Data
         if(!SuppressMessages) message("Data is assumed to be continuous")
         Continuous=TRUE
         dta=x
         y=x$y
         x=x$x 
         Dim=ncol(x)
       }
       else {#Discrete data
         if(!SuppressMessages) message("Data is assumed to be discrete")
         Continuous=FALSE
         dta=x
         x=matrix(1:4,2,2)#just some dummy numbers
         y=matrix(1:4,2,2)
         dta=list(x=dta[,"x"], y=dta[,"y"], vals_x=dta[,"vals_x"], vals_y=dta[,"vals_y"])
         DoTransform=FALSE
       }
    }
    else {
       Continuous=ifelse(any(is.na(c(vals_x, vals_y))), TRUE, FALSE)
       if(Continuous) {
          if(!SuppressMessages) message("Data is assumed to be continuous")
          dta=list(x=x, y=y)
          Dim=ncol(x)
       } 
       else {
          if(!SuppressMessages) message("Data is assumed to be discrete")
          dta=list(x=x, y=y, vals_x=vals_x, vals_y=vals_y)
          x=matrix(1:4,2,2)#just some dummy numbers
          y=matrix(1:4,2,2)
          DoTransform=FALSE
       }    
    }
    if(test_methods(doMethods, Continuous)) return(NULL)
    if(Continuous) {
      if(nrow(y)<nrow(x)) { #switch x and y so that nx>ny
        tmp=y
        y=x
        x=tmp
        dta=list(x=x, y=y)
      }
      rawdta=dta
      if(DoTransform) {
          dta=transform01(dta)
          x=dta$x
          y=dta$y
          Ranges=matrix(c(0, 1, 0, 1),2,2)
      }
    }
    if(missing(TSextra)) TSextra=list(aaa=0)
    if(Continuous) 
        TSextra = c(TSextra, 
                knn=function(x) FNN::get.knn(x, 5)$nn.index,
                dist=function(dta) find_dist(dta),
                distances=list(find_dist(dta)),
                DoTransform=DoTransform)
    else TSextra = c(TSextra, 
                     dist=function(dta) NULL,
                     organize=function(dta) dta=dta[order(dta[,1], dta[,2]), ],
                     samplingmethod=samplingmethod)
                     
    if(!missing(rnull)) {
       if(Continuous) 
           TSextra=c(TSextra, rnull=rnull, rawdta=list(rawdta))
       else TSextra=c(TSextra, rnull=rnull)
    }   
    outchi=list(statistics=NULL, p.value=NULL)
    outpvals=list(statistics=NULL, p.value=NULL)
# what methods are to be run?  
    CustomTS=TRUE
    if(missing(TS)) { # do included methods
      CustomTS=FALSE
      if(Continuous) {
          if(Dim==2) {
            if(length(nbins)==1) nbins=c(nbins, nbins)
            if(missing(rnull))
               outchi = chisq2D_test_cont(x,y, Ranges, nbins, minexpcount)
          }  
          if(missing(rnull))
             outpvals=TS_cont_pval(x, y) #Methods that find p values
          typeTS = 1
          TS=TS_cont
          dta=list(x=x, y=y)
      }
      else {
          typeTS=4
          if(missing(rnull))
             outchi = chisq2D_test_disc(dta, minexpcount)
          TS=TS_disc
      }    
    }  
    else { # do user-supplied tests
      if(substr(deparse(TS)[2], 1, 5)==".Call") {
         if(maxProcessor>1) {
            if(!SuppressMessages) message("Parallel Programming is not possible if custom TS is written in C++. Switching to single processor")  
            maxProcessor=1
         }  
      }
      if(Continuous) typeTS=length(formals(TS))
      else typeTS=ifelse(length(formals(TS))==5, 6, 5)  
    }
    TS_data=calcTS(dta, TS, typeTS, TSextra)
    if(B==0) return(TS_data)
    if(any(is.null(names(TS_data)))) {
      if(!SuppressMessages) message("output of TS routine has to be a named vector!")
      return(NULL)
    }
#  set number of processors for parallel programming
#  and do a timing check    
    if(missing(maxProcessor))
       maxProcessor=max(parallel::detectCores(logical = FALSE)-1, 1)  
    if(maxProcessor>1) {
      tm=timecheck(dta, TS, typeTS, TSextra)
      if(2*tm[1]*B<20 | B<2*maxProcessor) {
        maxProcessor=1
        if(!SuppressMessages) message("maxProcessor set to 1 for faster computation")
      }
      else if(!SuppressMessages) message(paste("Using ",maxProcessor," cores.."))  
    } 
# if either only one core is present, B=0 or maxProcessor=1, run testC. 
    if(!LargeSampleOnly) {
      if(maxProcessor==1) outTS = testC(dta, TS, typeTS, TSextra, B=B)
      else {
# run testC in parallel. Use one less core than is present, at most maxProcessor.    
        cl=parallel::makeCluster(maxProcessor)
        z=parallel::clusterCall(cl, testC, dta=dta, TS=TS, 
                              typeTS=typeTS, TSextra=TSextra, 
                              B=round(B/maxProcessor))
        parallel::stopCluster(cl)
# average p values over cores    
        p=z[[1]]$p.values
        for(i in 2:maxProcessor) p=p+z[[i]]$p.values
        p = round(p/maxProcessor, 4)  
        outTS = list(statistics=z[[1]]$statistics, p.values=p)
      }
    }
    else outTS=list(statistics=NULL, p.values=NULL)
    if(CustomTS) return(signif.digits(outTS))
    s = c(outTS$statistics, outpvals$statistics, outchi$statistic)
    p = c(outTS$p.values, outpvals$p.values, outchi$p.value)
    if(doMethods[1]!="all") {
      s=s[doMethods]
      p=p[doMethods]
    }  
    signif.digits(list(statistics=s, p.values=p))
}

#' test function
#' @param  dta data set
#' @param  TS test statistics
#' @param  typeTS format of TS
#' @param  TSextra additional info TS
#' @return Mean computation time
#' @export
timecheck=function(dta, TS, typeTS, TSextra) {
  f=function() calcTS(dta, TS, typeTS, TSextra)
  a=microbenchmark::microbenchmark(f(), 
              unit="seconds", times=10)
  out=c(as.numeric(summary(a)["mean"]), 0)
  if(typeTS==1 & !("rnull"%in%names(TSextra))) {
    f=function() TS_cont_pval(dta$x, dta$y)
    a=microbenchmark::microbenchmark(f(), 
              unit="seconds", times=5)
    out[2]=as.numeric(summary(a)["mean"])
  }
  out
}

#' Create case studies
#' 
#' This function creates the functions needed to run the various case studies.
#'
#' @param which name of the case study.
#' @param n =200, sample size for both data sets
#' @param nx =n, sample size for first data sets
#' @param ny =n, sample size for second data sets
#' @param nbins =-1, number of bins for chi-square test, 2D only
#' @param Ranges = matrix(c(-Inf, Inf, -Inf, Inf), 2, 2), ranges of variables
#' @param ReturnCaseNames =FALSE, should list of case studies be returned?
#' @return a list of functions and vectors
#' @export 
case.studies = function (which, n = 200, nx = n, ny = n, nbins = -1, Ranges = matrix(c(-Inf, 
    Inf, -Inf, Inf), 2, 2), ReturnCaseNames = FALSE) 
{
    cases = c("NormalD2", "tD2", "UniformMixtureD2", "FrankD2", 
        "ClaytonD2", "GumbelD2", "GalambosD2", "HuslerReissD2", 
        "ClaytonGumbelD2", "UniformFrankD2", "ParetoSimplexD2", 
        "KhoudrajiClaytonD2", "NormalUniformD2", "JoeD2", "DalitzCleoD2", 
        "DalitzPDGD2", "DalitzBabarD2", "NormalShiftM", "NormalStretchM", 
        "UniformRotateM", "UniformBetaM", "TruncExponentialM", 
        "DalitzCleoM", "DalitzPDGM", "DalitzBabarM", "FrankExponentialM", 
        "FrankLinearM", "FrankNormalM", "ClaytonExponentialM", 
        "ClaytonLinearM", "ClaytonNormalM", "GalambosExponentialM", 
        "GalambosLinearM", "GalambosNormalM", 
        "NormalD5", "tD5", "FrankD5", "ClaytonD5", "GumbelD5", "JoeD5",
        "UniformFrankD5", "FrankClaytonD5", "FrankJoeD5", "UniformExponentialM5",
        "FrankExponentialM5", "FrankLinearM5", "FrankNormalM5",
        "ClaytonExponentialM5", "ClaytonLinearM5", "ClaytonNormalM5")
    if (ReturnCaseNames) 
        return(cases)
    if (is.numeric(which)) 
        which = cases[which]
    uniforms = function(d=2) {
        x = matrix(stats::runif(d * nx), nx, d)
        y = matrix(stats::runif(d * ny), nx, d)
        out = list(x = x, y = y)
        if (nbins[1] > 0) 
            out = MD2sample::bincounterR(out, nbins = nbins, 
                Ranges = Ranges)
        out
    }
    if (which == "NormalD2") {
        return(list(f = function(a = 0) {
            S = diag(2)
            x = mvtnorm::rmvnorm(nx, sigma = S)
            S[1, 2] = a
            S[2, 1] = a
            y = mvtnorm::rmvnorm(ny, sigma = S)
            out = list(x = x, y = y)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 0.6)))
    }
    if (which == "tD2") {
        return(list(f = function(a = 0, df = 5, d = 2) {
            S = diag(d)
            x = mvtnorm::rmvt(2 * nx, sigma = S, df = df)
            x = x[abs(x[, 1]) < 5, ]
            x = x[abs(x[, 2]) < 5, ]
            x = x[1:nx, ]
            S[1, d] = a
            S[d, 1] = a
            y = mvtnorm::rmvt(2 * ny, sigma = S, df = df)
            y = y[abs(y[, 1]) < 5, ]
            y = y[abs(y[, 2]) < 5, ]
            y = y[1:ny, ]
            out = list(x = x, y = y)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 0.6)))
    }
    if (which == "UniformMixtureD2") {
        return(list(f = function(alpha = 0, d = 2) {
            if (alpha == 0) return(uniforms())
            x = matrix(stats::runif(d * nx), nx, d)
            m = round(alpha * ny)
            z1 = stats::runif(m)
            z2 = stats::rnorm(m, z1, 0.02)
            z = cbind(z2, 2 * z1 - z2)
            z[z[, 2] < 0, 2] = 0
            z[z[, 2] > 1, 2] = 1
            y = rbind(matrix(stats::runif(d * (ny - m)), ny - 
                m, d), z)
            out = list(x = x, y = y)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 0.35)))
    }
    if (which == "FrankD2") {
        return(list(f = function(theta = 0, d = 2) {
            if (theta == 0) return(uniforms())
            x = matrix(stats::runif(nx * d), ncol = d)
            cop = copula::frankCopula(theta, d)
            out = list(x = x, y = copula::rCopula(ny, cop))
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 3.15)))
    }
    if (which == "ClaytonD2") {
        return(list(f = function(theta = 0, d = 2) {
            if (theta == 0) return(uniforms())
            x = matrix(stats::runif(nx * d), ncol = d)
            cop = copula::claytonCopula(theta, d)
            out = list(x = x, y = copula::rCopula(ny, cop))
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 3)))
    }
    if (which == "GumbelD2") {
        return(list(f = function(theta = 0, d = 2) {
            if (theta == 0) return(uniforms())
            x = matrix(stats::runif(nx * d), ncol = d)
            cop = copula::gumbelCopula(theta, d)
            out = list(x = x, y = copula::rCopula(ny, cop))
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(1, 1.5)))
    }
    if (which == "GalambosD2") {
        return(list(f = function(theta = 0, d = 2) {
            if (theta == 0) return(uniforms())
            x = matrix(stats::runif(nx * d), ncol = d)
            cop = copula::galambosCopula(theta)
            out = list(x = x, y = copula::rCopula(ny, cop))
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 2.5)))
    }
    if (which == "HuslerReissD2") {
        return(list(f = function(theta = 0) {
            d = 2
            if (theta == 0) return(uniforms())
            x = matrix(stats::runif(nx * d), ncol = d)
            cop = copula::huslerReissCopula(theta)
            out = list(x = x, y = copula::rCopula(ny, cop))
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 2)))
    }
    if (which == "ClaytonGumbelD2") {
        return(list(f = function(a = 1/2) {
            cc <- copula::claytonCopula(copula::iTau(copula::claytonCopula(), 
                tau = 0.95))
            gc <- copula::gumbelCopula(copula::iTau(copula::gumbelCopula(), 
                tau = 0.15))
            wts1 <- c(1/2, 1/2)
            mcg1 <- copula::mixCopula(list(cc, gc), w = wts1)
            x <- copula::rCopula(nx, copula = mcg1)
            wts2 <- c(a, 1 - a)
            mcg2 <- copula::mixCopula(list(cc, gc), w = wts2)
            y <- copula::rCopula(nx, copula = mcg2)
            out = list(x = x, y = y)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0.5, 0.1)))
    }
    if (which == "UniformFrankD2") {
        return(list(f = function(a = 1/2) {
            un <- copula::indepCopula()
            fc <- copula::frankCopula(20, 2)
            wts <- c(1/2, 1/2)
            mcg <- copula::mixCopula(list(un, fc), w = wts)
            x <- copula::rCopula(nx, copula = mcg)
            wts <- c(a, 1 - a)
            mcg <- copula::mixCopula(list(un, fc), w = wts)
            y <- copula::rCopula(nx, copula = mcg)
            out = list(x = x, y = y)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0.5, 0.1)))
    }
    if (which == "ParetoSimplexD2") {
        return(list(f = function(th = 1, d = 2) {
            incBeta <- function(x, a, b) stats::pbeta(x, a, b) * 
                beta(a, b)
            psi = function(t, th) t^(-1/th) * incBeta(pmin(1, 
                t), a = 1/th, b = d)/th
            R = stats::runif(nx)^(-1)
            E = matrix(stats::rexp(nx * d), nrow = nx, ncol = d)
            S = E/matrix(rowSums(E), nrow = nx, ncol = d)
            x <- psi(R * S, 1)
            R = stats::runif(ny)^(-1/th)
            E = matrix(stats::rexp(ny * d), nrow = ny, ncol = d)
            S = E/matrix(rowSums(E), nrow = ny, ncol = d)
            y = psi(R * S, th)
            out = list(x = x, y = y)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(1, 1.3)))
    }
    if (which == "KhoudrajiClaytonD2") {
        return(list(f = function(a) {
            kcx <- copula::khoudrajiCopula(copula2 = copula::claytonCopula(6), 
                shapes = c(0.2, 0.95))
            kcy <- copula::khoudrajiCopula(copula2 = copula::claytonCopula(6), 
                shapes = c(a, 0.95))
            out = list(x = copula::rCopula(nx, copula = kcx), 
                y = copula::rCopula(ny, copula = kcy))
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0.2, 0.7)))
    }
    if (which == "NormalUniformD2") {
        return(list(f = function(a = 1/2) {
            nc1 <- copula::indepCopula()
            nc2 <- copula::normalCopula(copula::iTau(copula::normalCopula(), 
                tau = 0.5))
            wts <- c(1, 0)
            mcg <- copula::mixCopula(list(nc1, nc2), w = wts)
            x <- copula::rCopula(nx, copula = mcg)
            wts <- c(1 - a, a)
            mcg <- copula::mixCopula(list(nc1, nc2), w = wts)
            y <- copula::rCopula(nx, copula = mcg)
            out = list(x = x, y = y)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 0.7)))
    }
    if (which == "JoeD2") {
        return(list(f = function(a = 1) {
            nc1 <- copula::indepCopula()
            nc2 = nc1
            if (a > 1) nc2 = copula::joeCopula(a)
            out = list(x = copula::rCopula(nx, copula = nc1), 
                y = copula::rCopula(ny, copula = nc2))
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(1, 2)))
    }
    if (startsWith(which, "Dalitz")) {
        par2 = 130
        addon = "Diagonal Stripe"
        if (endsWith(which, "M")) {
            par2 = 35
            addon = c("Left Stripe", "Bottom Stripe", "Right Stripe")
        }
        experiment = 1
        if (startsWith(which, "DalitzPDG")) 
            experiment = 2
        if (startsWith(which, "DalitzBabar")) 
            experiment = 3
        return(list(f = function(a = 0) {
            tmp = MD2sample::rDalitz(a, experiment, addon, , 
                nx = 2 * nx, ny = 2 * ny)
            xl = range(tmp[[3]][, 1])
            yl = range(tmp[[3]][, 2:3])
            Ranges = matrix(c(xl, yl), 2, 2)
            x = tmp[[1]]
            x = x[!is.na(x[, 1]), ]
            x = x[!is.na(x[, 2]), ]
            y = tmp[[2]]
            y = y[sample(1:nrow(y)), ]
            y = y[!is.na(y[, 1]), ]
            y = y[!is.na(y[, 2]), ]
            out = list(x = x[1:nx, ], y = y[1:ny, ])
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, par2)))
    }

    if (which == "NormalShiftM") {
        return(list(f = function(mu = 0) {
            x = mvtnorm::rmvnorm(nx, c(0, 0))
            y = mvtnorm::rmvnorm(ny, mean = c(0, mu))
            out = list(x = x, y = y)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 0.5)))
    }
    if (which == "NormalStretchM") {
        return(list(f = function(s = 1) {
            x = mvtnorm::rmvnorm(nx, sigma = diag(c(1, 1)))
            y = mvtnorm::rmvnorm(ny, sigma = diag(c(1, s)))
            out = list(x = x, y = y)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(1, 2)))
    }
    if (which == "UniformRotateM") {
        return(list(f = function(alpha = 0) {
            x = matrix(stats::runif(2 * nx), ncol = 2)
            y = matrix(stats::runif(2 * ny), ncol = 2) %*% 
              matrix(c(cos(alpha), sin(alpha), -sin(alpha), 
                       cos(alpha)), 2, 2)
            out = list(x = x, y = y)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 0.175)))
    }
    if (which == "UniformBetaM") {
        return(list(f = function(a = 1) {
            x = matrix(stats::runif(2 * nx), ncol = 2)
            y = matrix(stats::rbeta(2 * ny, a, a), ncol = 2)
            out = list(x = x, y = y)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(1, 1.7)))
    }
    if (which == "TruncExponentialM") {
        return(list(f = function(lambda = 1) {
            tmp = NULL
            repeat {
                tmp = c(tmp, stats::rexp(4 * nx, 1))
                tmp = tmp[tmp < 2]
                if (length(tmp) > 2 * nx) break
            }
            x = matrix(tmp[1:(2 * nx)], ncol = 2)
            tmp = NULL
            repeat {
                tmp = c(tmp, stats::rexp(4 * nx, lambda))
                tmp = tmp[tmp < 2]
                if (length(tmp) > 2 * ny) break
            }
            y = matrix(tmp[1:(2 * ny)], ncol = 2)
            out = list(x = x, y = y)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(1, 1.5)))
    }
    if (which == "FrankExponentialM") {
        return(list(f = function(lambda = 1) {
            cop = copula::frankCopula(3, 2)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Exponential", 
                lambda)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(1, 1.7)))
    }
    if (which == "FrankLinearM") {
        return(list(f = function(s = 0) {
            cop = copula::frankCopula(3, 2)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Linear", 
                s)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 0.5)))
    }
    if (which == "FrankNormalM") {
        return(list(f = function(s = 1) {
            cop = copula::frankCopula(3, 2)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Normal", 
                s)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(1, 1.5)))
    }
    if (which == "ClaytonExponentialM") {
        return(list(f = function(lambda = 1) {
            cop = copula::claytonCopula(3, 2)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Exponential", 
                lambda)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(1, 1.7)))
    }
    if (which == "ClaytonLinearM") {
        return(list(f = function(s = 0) {
            cop = copula::claytonCopula(3, 2)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Linear", 
                s)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 0.5)))
    }
    if (which == "ClaytonNormalM") {
        return(list(f = function(s = 1) {
            cop = copula::claytonCopula(3, 2)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Normal", 
                s)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(1, 1.5)))
    }
    if (which == "GalambosExponentialM") {
        return(list(f = function(lambda = 1) {
            cop = copula::galambosCopula(2.5)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Exponential", 
                lambda)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(1, 1.4)))
    }
    if (which == "GalambosLinearM") {
        return(list(f = function(s = 0) {
            cop = copula::galambosCopula(2.5)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Linear", 
                s)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(0, 0.65)))
    }
    if (which == "GalambosNormalM") {
        return(list(f = function(s = 1) {
            cop = copula::galambosCopula(2.5)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Normal", 
                s)
            if (nbins[1] > 0) out = MD2sample::bincounterR(out, 
                nbins = nbins, Ranges = Ranges)
            out
        }, nbins = c(5, 5), param_alt = c(1, 1.42)))
    }
    if (which == "NormalD5") {
        return(list(f = function(a = 0) {
            S = diag(5)
            x = mvtnorm::rmvnorm(nx, sigma = S)
            S = matrix(a, 5, 5)
            for (i in 1:5) S[i, i] = 1
            y = mvtnorm::rmvnorm(ny, sigma = S)
            list(x = x, y = y)
        }, param_alt = c(0, 0.3)))
    }
    if (which == "tD5") {
        return(list(f = function(a = 0, df = 5) {
            S = diag(5)
            x = mvtnorm::rmvt(nx, sigma = S, df = df)
            S = matrix(a, 5, 5)
            for (i in 1:5) S[i, i] = 1
            y = mvtnorm::rmvt(ny, sigma = S, df = df)
            list(x = x, y = y)
        }, param_alt = c(0, 0.3)))
    }
    if (which == "FrankD5") {
        return(list(f = function(theta = 0, d = 5) {
            x = matrix(stats::runif(nx * d), ncol = d)
            if (theta == 0) return(list(x = x, y = matrix(stats::runif(ny * 
                d), ncol = d)))
            cop = copula::frankCopula(theta, d)
            list(x = x, y = copula::rCopula(ny, cop))
        }, param_alt = c(0, 1.7)))
    }
    if (which == "ClaytonD5") {
        return(list(f = function(theta = 0, d = 5) {
            x = matrix(stats::runif(nx * d), ncol = d)
            if (theta == 0) return(list(x = x, y = matrix(stats::runif(ny * 
                d), ncol = d)))
            cop = copula::claytonCopula(theta, d)
            list(x = x, y = copula::rCopula(ny, cop))
        }, param_alt = c(0, 0.5)))
    }
    if (which == "GumbelD5") {
        return(list(f = function(theta = 1, d = 5) {
            x = matrix(stats::runif(nx * d), ncol = d)
            if (theta == 1) return(list(x = x, y = matrix(stats::runif(ny * 
                d), ncol = d)))
            cop = copula::gumbelCopula(theta, d)
            list(x = x, y = copula::rCopula(ny, cop))
        }, param_alt = c(1, 1.25)))
    }
    if (which == "JoeD5") {
        return(list(f = function(a = 1) {
            nc1 <- copula::indepCopula(dim = 5L)
            nc2 = nc1
            if (a > 1) nc2 = copula::joeCopula(a, dim = 5L)
            list(x = copula::rCopula(nx, copula = nc1), y = copula::rCopula(ny, 
                copula = nc2))
        }, param_alt = c(1, 2)))
    }    
    if (which == "UniformFrankD5") {
        return(list(f=function(a=0) { 
          nc1 <- copula::indepCopula(dim=5) # the first component
          nc2 <- copula::frankCopula(5,dim=5) # the second component
          wts <- c(1, 0) # the corresponding weights
          mcg <- copula::mixCopula(list(nc1, nc2), w = wts)
          x <- copula::rCopula(nx, copula = mcg)
          wts <- c(1-a, a) # the corresponding weights
          mcg <- copula::mixCopula(list(nc1, nc2), w = wts)
          y <- copula::rCopula(ny, copula = mcg)
          out=list(x=x, y=y)
          out
      }, param_alt = c(0, 0.3)))
    }
    if (which == "FrankClaytonD5") {
        return(list(f=function(a=0) { 
          nc1 <- copula::frankCopula(5, dim=5L) # the first component
          nc2 <- copula::claytonCopula(5,dim=5L) # the second component
          wts <- c(1, 0) # the corresponding weights
          mcg <- copula::mixCopula(list(nc1, nc2), w = wts)
          x <- copula::rCopula(nx, copula = mcg)
          wts <- c(1-a, a) # the corresponding weights
          mcg <- copula::mixCopula(list(nc1, nc2), w = wts)
          y <- copula::rCopula(ny, copula = mcg)
          out=list(x=x, y=y)
          out
      }, param_alt = c(0, 0.5)))
    }
    if (which == "FrankJoeD5") {
        return(list(f=function(a=0) { 
          nc1 <- copula::frankCopula(5, dim=5L) # the first component
          nc2 <- copula::joeCopula(3, dim=5L) # the second component
          wts <- c(1, 0) # the corresponding weights
          mcg <- copula::mixCopula(list(nc1, nc2), w = wts)
          x <- copula::rCopula(nx, copula = mcg)
          wts <- c(1-a, a) # the corresponding weights
          mcg <- copula::mixCopula(list(nc1, nc2), w = wts)
          y <- copula::rCopula(ny, copula = mcg)
          out=list(x=x, y=y)
          out
      }, param_alt = c(0, 1)))
    }
    if (which == "FrankExponentialM5") {
        return(list(f = function(s = 1) {
            cop = copula::frankCopula(3, 5)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Exponential", s)
            out
        }, param_alt = c(1, 1.4)))
    }
    if (which == "FrankLinearM5") {
        return(list(f = function(s = 0) {
            cop = copula::frankCopula(3, 5)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Linear", s)
            out
        }, param_alt = c(0, 0.4)))
    }
    if (which == "FrankNormalM5") {
        return(list(f = function(s = 1) {
            cop = copula::frankCopula(3, 5)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Normal", s)
            out
        }, param_alt = c(1, 1.25)))
    }
     if (which == "ClaytonExponentialM5") {
        return(list(f = function(s = 1) {
            cop = copula::frankCopula(3, 5)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Exponential", s)
            out
        }, param_alt = c(1, 1.4)))
    }
    if (which == "ClaytonLinearM5") {
        return(list(f = function(s = 0) {
            cop = copula::frankCopula(3, 5)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Linear", s)
            out
        }, param_alt = c(0, 0.4)))
    }
    if (which == "ClaytonNormalM5") {
        return(list(f = function(s = 1) {
            cop = copula::frankCopula(3, 5)
            x = copula::rCopula(nx, cop)
            y = copula::rCopula(ny, cop)
            out = list(x = x, y = y)
            out = MD2sample::change.marginals(out, "Normal", s)
            out
        }, param_alt = c(1, 1.25)))
    }   
    if (which == "UniformExponentialM5") {
      return(list(f = function(s = 1) {
        x = matrix(stats::runif(5 * nx), nx, 5)
        y = matrix(stats::runif(5 * nx), nx, 5)
        for(i in 1:5) {
          x[,i]=qexp(x[,i], 1)
          y[,i]=qexp(y[,i], s)
        }
        out = list(x = x, y = y)
      }, param_alt = c(1, 1.175)))
    }
}

#' Count Events in Bins
#' 
#' This function discretizes continuous data in two dimensions
#' 
#' @param x a matrix of numbers or a list 
#' @param y a matrix of numbers 
#' @param nbins =c(5,5) number of bins in x and in y direction
#' @param  Ranges =matrix(c(-Inf, Inf, -Inf, Inf),2,2) a 2x2 matrix with lower and upper bounds
#' @keywords internal
#' @return a list counts and vals
#' @export
bincounterR=function(x, y, nbins=c(5, 5), Ranges =matrix(c(-Inf, Inf, -Inf, Inf),2,2)) {
  if(length(nbins)==1) nbins=c(nbins, nbins)
  if(is.list(x)) {
     dta=x
     x=dta$x
     y=dta$y
  }
  else dta=list(x=x, y=y)
  if(ncol(x)!=2) {message("This is for two dimensional data only!");return(NULL)}
  xy=rbind(x, y)
  low=Ranges[1, ]
  if(!is.finite(low[1])) low[1]=min(xy[,1])-1e-5
  if(!is.finite(low[2])) low[2]=min(xy[,2])-1e-5
  high=Ranges[2, ]
  if(!is.finite(high[1])) high[1]=max(xy[,1])+1e-5
  if(!is.finite(high[2])) high[2]=max(xy[,2])+1e-5
  grd=as.list(1:2)
  for(i in 1:2) { # equal space grid
      grd[[i]]=seq(low[i], high[i], length=nbins[i]+1)
  }  
  midpoints_x=round((grd[[1]][-1]+grd[[1]][-(nbins[1]+1)])/2,2)
  midpoints_y=round((grd[[2]][-1]+grd[[2]][-(nbins[2]+1)])/2,2)
  vals=cbind(rep(midpoints_x, nbins[2]),rep(midpoints_y, each=nbins[1]))
  x=rbind(x, vals)
  y=rbind(y, vals)
  C_x=0*x
  C_y=0*y
  for(i in 1:2) {
    C_x[,i]=cut(x[,i], grd[[i]], labels=1:nbins[i],include.lowest =TRUE)
    C_y[,i]=cut(y[,i], grd[[i]], labels=1:nbins[i],include.lowest =TRUE)
  }
  tmp_x=table(C_x[,1],C_x[,2])-1
  tmp_y=table(C_y[,1],C_y[,2])-1  
  out=matrix(0, prod(nbins), 4)
  out[, 1:2]=vals
  for(i in 1:nbins[2]) {
     out[(i-1)*nbins[1]+1:nbins[1] , 3]=tmp_x[ ,i]
     out[(i-1)*nbins[1]+1:nbins[1] , 4]=tmp_y[ ,i]
  }
  colnames(out)=c("vals_x", "vals_y", "x", "y")
  out[order(out[,1], out[,2]), ]
}

#' Chi-square test for continuous data, example of user supplied function
#' 
#' This function does the chi square two sample test for continuous data in two dimensions
#' 
#' @param x  a matrix of numbers 
#' @param y a matrix of numbers 
#' @param TSextra list with some info
#' @return either a test statistic or a p value
#' @export
chiTS.cont=function(x, y, TSextra) {
  nbins=TSextra$nbins
  if(!is.matrix(nbins)) nbins=rbind(nbins)
  chi=rep(0, nrow(nbins))
  pval=rep(0, nrow(nbins))
  for(k in 1:nrow(nbins)) {
    p1=quantile(c(x[,1],y[,1]), (0:nbins[k,1])/nbins[k,1])
    p2=quantile(c(x[,2],y[,2]), (0:nbins[k,2])/nbins[k,2])
    Ox=Oy=matrix(0, nbins[k,1], nbins[k,2])
    for(i in 1:nbins[k,1]) {
      x1=x[x[,1]>=p1[i] & x[,1]<=p1[i+1], ]  
      y1=y[y[,1]>=p1[i] & y[,1]<=p1[i+1], ]
      for(j in 1:nbins[k,2]) {
        x2=x1[x1[,2]>=p2[j] & x1[,2]<=p2[j+1], ,drop=FALSE]  
        y2=y1[y1[,2]>=p2[j] & y1[,2]<=p2[j+1], ,drop=FALSE]
        Ox[i,j]=nrow(x2)
        Oy[i,j]=nrow(y2)
      }
    }
    Oxy=Ox+Oy
    I=(Oxy>3)    
    s=sqrt(sum(Ox[I])/sum(Oy[I]))
    chi[k]=sum((Ox[I]/s-s*Oy[I])^2/Oxy[I])
    df=length(Ox[I])-1
    pval[k]=1-stats::pchisq(chi[k],df)
    names(chi)[k]=paste0("Chisq Stat ", nbins[k,1], ",", nbins[k,2])
    names(pval)[k]=paste0("Chisq Pval ", nbins[k,1], ",", nbins[k,2])
  }  
  if(TSextra$which=="All") return(list(statistic=chi, p.value=pval, df=df))
  if(startsWith(TSextra$which, "stat")) return(chi)
  pval
}

#' Chi-square test for discrete data, example of user supplied function
#' 
#' This function does the chi square two sample test for discrete data in two dimensions
#' 
#' @param x  a vector with counts 
#' @param y  a vector with counts 
#' @param vals_x  a vector with points 
#' @param vals_y  a vector with points 
#' @param TSextra list with some info
#' @return either a test statistic or a p value
#' @export
chiTS.disc=function(x, y, vals_x, vals_y, TSextra) {
  I=(x+y>0)
  s=sqrt(sum(x[I])/sum(y[I]))
  chi=sum((x[I]/s-s*y[I])^2/(x+y)[I])
  names(chi)="Chisq Stat"
  df=length(x[I])-1
  if(startsWith(TSextra$which, "stat")) return(chi)
  pval=1-stats::pchisq(chi,df)
  names(pval)="Chisq P"
  pval
}

#' Friedman-Rafsky (FR) test
#'
#' FR test is a multivariate generalization of nonparametric two-sample test. This function is an implementation
#' with customized options, including a visualization of the minimum spanning tree (MST).
#'
#' @param samp1 Numeric matrix or data frame for Sample 1. Rows are multivariate dimensions, and columns are samples. E.g. gene by cell.
#' @param samp2 Numeric matrix or data frame for Sample 2.
#' @param use.cosine An option if to use cosine distance. Logical variable. By default (\code{FALSE}),
#' Euclidean distance is used.
#' @param binary An option if to use binary values. Logical variable. Default: \code{FALSE}. If \code{TRUE},
#' use \code{binary.cutoff} to dichotomize \code{samp1} and \code{samp2}.
#' @param binary.cutoff Numeric value for binary cutoff. Binary value = 1 if greater than \code{binary.cutoff}, 0 otherwise. Default: \code{2}.
#' @param plot.MST Boolean variable indicating if to plot the minimum spanning tree (MST). Default: \code{FALSE}.
#' @param col Character vector of length two for customized colors of the nodes in MST. Default: \code{c("#F0E442", "#56B4E9")}.
#' @param label.names Character vector of length two for customized names of the two samples. Default: \code{c("Sample 1","Sample 2")}.
#' @param vertex.size,edge.width,... Additional plotting parammeters passed to \code{\link[igraph]{plot.igraph}}. Default: \code{vertex.size=5, edge.width=1}.
#'
#' @return Test statistics and p-values.
#' \item{runs}{Total number of subtrees.}
#' \item{runs.samp1}{Number of subtrees of Sample 1.}
#' \item{runs.samp2}{Number of subtrees of Sample 2.}
#' \item{stat}{The standardized FR statistic.}
#' \item{p.value}{P-value of the FR test.}
#' @export

FR.test <- function(samp1, samp2,
                   ## two options: 1. cosine distance, 2. use binary values
                   use.cosine=FALSE, binary=FALSE, binary.cutoff=2,
                   ## plot minimum spanning tree
                   plot.MST=FALSE, col=c("#F0E442", "#56B4E9"), label.names=c("Sample 1","Sample 2"),
                   vertex.size=5, edge.width=1,
                   ...)
{
  ## data input matrices: rows = multivariate dimensions, columns = samples
  samp1 <- t(samp1)
  samp2 <- t(samp2)
  xx <- as.matrix(samp1)
  yy <- as.matrix(samp2)
  
  ## get sample sizes
  m <- max(ncol(xx), 1)
  n <- max(ncol(yy), 1)
  N <- m+n
  
  ##--- OPTION 2: BINARY VALUES ---##
  if(binary){
    xx <- matrix(as.numeric(xx>binary.cutoff), nrow=nrow(xx))
    yy <- matrix(as.numeric(yy>binary.cutoff), nrow=nrow(yy))
  }
  ##---##
  
  ## column-wise combined data matrix
  dat <- cbind(xx, yy)
  
  ##--- OPTION 1: CONSINE DISTANCE ---##
  if(use.cosine){
    cosmat <- lsa::cosine(dat)
    cosmat[is.na(cosmat)] <- -1 #if one point is at the origin, set the -1,0,1??? cosine value
    distobj <- stats::as.dist(1-cosmat) #large cosine, small distance; and vise versa
  }
  ##---##
  
  ## euclidean distance and MST
  else distobj <- stats::dist(t(dat), method="euclidean", diag=T, upper=T)
  myMST <- ade4::neig2mat(ade4::mstree(distobj))
  
  
  ## count total runs (i.e. total subgraphs)
  bottomleft <- myMST[(m+1):N,1:m]
  runs <- sum(bottomleft)+1
  
  ## count subgraphs for each sample
  bottomright <- myMST[(m+1):N,(m+1):N]
  g.samp2 <- igraph::graph_from_adjacency_matrix(bottomright, mode="upper", weighted=NULL, diag=FALSE)
  runs.samp2 <- igraph::components(g.samp2)$no
  topleft <- myMST[1:m,1:m]
  g.samp1 <- igraph::graph_from_adjacency_matrix(topleft, mode="upper", weighted=NULL, diag=FALSE)
  runs.samp1 <- igraph::components(g.samp1)$no
  ## check
  # runs==sum(runs.samp1, runs.samp2)
  
  ## calculate common nodes
  xsum <- colSums(myMST)
  C <- sum(xsum*(xsum-1))/2
  ## calculate mean and variance
  mu <- 2*m*n/N + 1
  sigma.sq <- (2*m*n/(N*(N-1)))*((2*m*n-N)/N+(C-N+2)*(N*(N-1)-4*m*n+2)/((N-2)*(N-3)))
  ## the standardized FR-stat and p-value
  stat <- (runs-mu)/sqrt(sigma.sq)
  p.value <- stats::pnorm(stat)
  
  ## output
  return(list("statistic"=stat, "p.value"=p.value))
}

---
title: "MD2sample"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{MD2sample}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: [MD2sample.bib]  
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(MD2sample)
BT=100  #number of simulations for testing
BP=50  #number of simulations for power estimation
# These are set very low to satisfy CRAN submission rules
```

**Note** that all examples are run without parallel processing and a small number of simulation runs B to satisfy CRAN submission rules.

This package brings together a number of routines for the two sample problem for multivariate data. There are two data sets x and y and we want to test whether they were generated by the same probability distribution. 

The highlights of this package are:

-  it runs a large number of tests simultaneously.  
-  the user can provide their own tests  
-  it works for continuous and for discrete data.  
-  some  tests use the permutation method to find p values, others use some large sample theory.   
-  the user can also provide a routine to generate simulated data for the goodness-of-fit/two-sample hybrid problem.  
-  tests can be combined and a single adjusted p value found  
-  it uses Rcpp and parallel programming to be very fast.  
-  it includes routines that make power studies very simple.  
-  it has a routine that runs (up to) 50 different case studies, comparing the power of a new method to those included in the package. This will make it very easy for someone who has developed a new method to compare its performance vs. those included in the package.

## Continuous Data

### Example 1 

We generate two-dimensional data sets of size 100 and 120 respectively from multivariate normal distributions and run the test:

```{r}
set.seed(123)
```

```{r}
x1 = mvtnorm::rmvnorm(100, c(0,0))
y1 = mvtnorm::rmvnorm(120, c(0,0))
twosample_test(x1, y1, B=BT, maxProcessor = 1)
```


**Arguments of twosample_test**

-  *x* a matrix of numbers with different observations in different rows,  or a list of x and y.  
-  *y* a matrix of numbers, or missing if x is a list with both data sets.  
-  *TS* a routine to calculate test statistics or p values for new tests.  
-  *TSextra* additional info passed to TS, if necessary.  
-  *B=5000* number of simulation runs for permutation test.  
-  *nbins=c(5,5)* bins for chi square tests (2D only).  
-  *minexpcount=5* lowest required count for chi-square test.  
-  *Ranges=matrix(c(-Inf, Inf, -Inf, Inf),2,2)* a 2x2 matrix with lower and upper bounds of observations.  
-  *rnull* routine that generates data under the null hypothesis, for hybrid problem (see below). 
-  *DoTransform=TRUE* should data be transfomed to unit hypercube?  
-  *SuppressMessages=FALSE* should informative messages be printed? 
-  *maxProcessor* maximum number of cores to use. If set to 1 no parallel processing is used.  
-  *doMethods="all"* names of method to include.


### New Tests

The routine *chiTS.cont* (included in the package) finds either the test statistic or the p value of a chi square test in two dimensions. First we need to set

```{r}
TSextra=list(which="statistic", nbins=cbind(c(3,3), c(3,4)))
```

this list is passed to *chiTS.cont* and tells the routine to

- return the test statistic (and not the p value)  
- Run two tests, first with 3 and 3 bins in the x and y direction and then 3 and 4 bins.  


```{r}
twosample_test(x1, y1, TS=chiTS.cont, TSextra=TSextra, 
               B=BT, maxProcessor = 1)
```

**Arguments and output of new test routine for continuous data**

The arguments have to be x and y for the two data sets and (optionally) a list called *TSextra* for any additional input needed to find test statistic.

Note that the output vector of the routine has to be a **named** vector. 
If the routine is written in Rcpp parallel programming is not available.

### Adjusted p values

If several tests are run one can use the routine *twosample_test_adjusted_pvalue* to find a p value that is adjusted for simultaneous testing:

```{r}
twosample_test_adjusted_pvalue(x1, y1, B=c(BT, BT),
                               maxProcessor = 1)
```

### Power Estimation

The routine **twosample_power** allows us to estimate the power of the tests. 

### Example 2

Let's say we want to estimate the power when one data set comes from a multivariate standard normal distribution and the other from a normal distribution with a different covariance. We first need a function that generates these data sets:

```{r}
f=function(a=0) {
   S=diag(2) 
   x=mvtnorm::rmvnorm(100, sigma = S)
   S[1,2]=a
   S[2,1]=a
   y=mvtnorm::rmvnorm(120, sigma = S)
   list(x=x, y=y)
}   
```

Now we can run

```{r}
twosample_power(f, c(0,0.5), B=BP, maxProcessor=1)
```

**Arguments of twosample_power**

-  *f* a function that generates data
-  *...* arguments passed to f, up to 2.   
-  *TS* a routine to calculate test statistics or p values for new tests.  
-  *TSextra* additional info passed to TS, if necessary.
-  *alpha=0.05* desired type I error probability.  
-  *B=1000* number of simulation runs for power estimation.  
-  *nbins=c(5,5)* bins for chi square tests (2D only).  
-  *minexpcount=5* lowest required count for chi-square test.  
-  *Ranges=matrix(c(-Inf, Inf, -Inf, Inf),2,2)* a 2x2 matrix with lower and upper bounds of observations.  
-  *rnull* routine that generates data under the null hypothesis, for hybrid problem (see below).  
-  *DoTransform=TRUE* should data be transformed to unit hypercube?  
-  *SuppressMessages=FALSE* should informative messages be printed?  
-  *LargeSampleOnly=FALSE* should only methods with large sample theories be run?  
-  *doMethods="all"* names of method to include.


Again the user can provide their own routine:

```{r}
twosample_power(f, c(0,0.5), TS=chiTS.cont, 
                TSextra=TSextra, B=BP, maxProcessor=1)
```

The routine that generates data can also have two arguments:

```{r}
f1=function(a=0, n=100) {
   S=diag(2) 
   x=mvtnorm::rmvnorm(100, sigma = S)
   S[1,2]=a
   S[2,1]=a
   y=mvtnorm::rmvnorm(n, sigma = S)
   list(x=x, y=y)
}   
```

If the user routine returns p values run

```{r}
TSextra=list(which="pvalue", nbins=c(5,5))
twosample_power(f1, c(0,0.5), c(100, 120), 
                TS=chiTS.cont, TSextra=TSextra, B=BP,
                With.p.value=TRUE, maxProcessor=1)
```


## Discrete Data

First note that tests for discrete data are implemented only in two dimensions. 

### Example 3

We consider the case of data from binomial distributions:

```{r}
vals_x = 0:5 #possible values of first random variable
vals_y = 0:6 #possible values of second random variable
a1x = rbinom(1000, 5, 0.5) 
a2x = rbinom(1000, 6, 0.5)
a1y = rbinom(1200, 5, 0.5) 
a2y = rbinom(1200, 6, 0.5)
x2=matrix(0, 6*7, 4)
colnames(x2)=c("vals_x", "vals_y", "x", "y")
x2[, 1]=rep(vals_x, length(vals_y))
x2[, 2]=rep(vals_y, each=length(vals_x))
for(i in 0:5) {
  for(j in 0:6) {
    x2[x2[,1]==i&x2[,2]==j, 3]=sum(a1x==i&a2x==j)
    x2[x2[,1]==i&x2[,2]==j, 4]=sum(a1y==i&a2y==j)
  }
}
twosample_test(x2, B=BT, maxProcessor = 1)
```

**Arguments of twosample_test**

As in the case of continuous data the arguments include *TS*, *TSextra*, *minexpcount*, *rnull*, *maxProcessor* and *doMethods*. In addition we now have

-  *x* either a vector of counts for first variable, or a matrix with columns vals_x, vals_y, x and y.  
- *y* vector with counts of second variable  
- *vals_x* possible values of first variable  
- *vals_y* possible values of second variable  
- *samplingmethod ="Binomial"* how the permuted data is found. The alternative is "independence", which is slightly more accurate but also much slower. 

### Adjusted p values

Again one can find a p value adjusted for simultaneous testing:

```{r}
twosample_test_adjusted_pvalue(x2, B=c(BT, BT), maxProcessor=1)
```

### User supplied tests

The routine *chiTS.disc* (included in package) does a chi-square test for discrete data:

```{r}
TSextra=list(which="statistic")
twosample_test(x2, TS=chiTS.disc, TSextra=TSextra,
               B=BT, maxProcessor = 1)
```


### Power Estimation

Again we need a routine that generates data sets. In the discrete case this has to be a routine whose output is a matrix with columns named vals_x, vals_y, x and y

```{r}
g=function(a=0) {
   x=cbind(rbinom(1000, 5, 0.5), rpois(1000, 1))
   x[,2][x[,2]>5]=5
   lambda=1+a*x[,1]/5
   y=cbind(rbinom(1200, 5, 0.5), rpois(1200, lambda))
   y[,2][y[,2]>5]=5
   vx=0:5
   vy=0:5
   A=matrix(0,length(vx)*length(vy),4)                  
   k=0
   for(i in seq_along(vx))
     for(j in seq_along(vy)) {
       k=k+1
       A[k,1]=vx[i]
       A[k,2]=vy[j]
       A[k,3]=sum(x[,1]==vx[i] & x[,2]==vy[j])
       A[k,4]=sum(y[,1]==vx[i] & y[,2]==vy[j])
     }
   colnames(A)=c("vals_x", "vals_y", "x", "y")
   A
}
```

```{r}
twosample_power(g, c(0, 0.25, 0.5), B=BP, maxProcessor=1)
```

or using a user supplied test:

```{r}
TSextra=list(which="statistic")
twosample_power(g, c(0, 0.25, 0.5), B=BP, 
                TS=chiTS.disc, TSextra=TSextra, maxProcessor=1)
TSextra=list(which="pvalue")
twosample_power(g, c(0, 0.25, 0.5), B=BP, 
    TS=chiTS.disc, TSextra=TSextra, With.p.value = TRUE,
    maxProcessor=1)
```

## Goodness-of-fit/Two-sample hybrid problem

We have a data set x and we want to test whether it comes from a bivariate normal distribution. Instead of a goodness-of-fit test, however, we generate a Monte Carlo data set y from a bivariate normal rv with mean and covariance estimated from the real data set x, and then run a two-sample test.

In this scenario the two data sets are not independent, and the permutation approach to finding p values is extremely conservative, that is, the true type I error probability is much smaller than the nominal one. This in turn makes the power of the test much lower as well. Instead one can supply a routine that generates new data, just as one would in a goodness-of-fit test. Moreover, all the methods who find their own p values will now fail completely and so should not be run.

```{r}
# generate real and MC data sets:
f=function(mu) {
    x=mvtnorm::rmvnorm(100, c(mu, mu))
    y=mvtnorm::rmvnorm(100, apply(x, 2, mean), cor(x))
    list(x=x, y=y)
}
#True data is a mixture of normal and uniform
g=function(alpha=0) {
    x=rbind(mvtnorm::rmvnorm((1-alpha)*100, c(0, 0)),
            matrix(runif(200*alpha),ncol=2))
    y=mvtnorm::rmvnorm(100, apply(x, 2, mean), cor(x))
    list(x=x, y=y)
}
# generate two-sample data set
rnull=function(dta) {
   x=mvtnorm::rmvnorm(nrow(dta$x), apply(dta$x, 2, mean), cor(dta$x))
   y=mvtnorm::rmvnorm(nrow(x), apply(x, 2, mean), cor(x))
   list(x=x, y=y)
}
```

```{r Hybrid}
# Only run these methods for hybrid problem
mt=c("KS", "K", "CvM", "AD", "NN1", "NN5", "AZ", "BF", "BG", "MMD")
# Null hypothesis is true:
twosample_power(f, c(0, 1), doMethods = mt, B=BP, maxProcessor = 1)
twosample_power(f, c(0, 1), rnull=rnull, B=BP, maxProcessor = 1)
# Null hypothesis is false:
twosample_power(g, c(0, 0.5), doMethods = mt, B=BP, maxProcessor = 1)
twosample_power(g, c(0, 0.5), rnull=rnull, B=BP, maxProcessor = 1)
```

As we can see, the true type I error using the permutation method is much smaller than the nominal $\alpha=0.05$, and so the power is lower as well.

## Benchmarking 

The routine *run.studies* allows the user to quickly compare the power of a new test with the power of the included tests for 25 different combinations of null hypothesis vs alternative for continuous data and 20 for discrete data. It also allows the user to find the power for those case studies for different sample size and type I error probabilities.

As an example, let's compare the power of the test based on differences in means to the included ones for two of the studies.

Note that these examples are not run so the package can be submitted to CRAN:


```{r eval=FALSE}
run.studies(Continuous=TRUE, 
            TS=newTest_1, 
            study=c("NormalD2", "tD2"), 
            B=1000)
```

**Arguments of run.studies**

-  *study* either the name of the study, or its number. If missing all the studies are run.  
-  *Continuous=TRUE* run cases for continuous data.  
-  *TS* routine to calculate test statistics.  
-  *TSextra* list passed to TS, if any.  
-  *With.p.value=FALSE* does user supplied routine return p values?  
-  *nsample=200* desired sample size.  
-  *alpha=0.05*  type I error.  
-  *param_alt* vector or matrix of values of parameters under the alternative hypothesis. If missing included values are used.  
-  *SuppressMessages=FALSE* should messages be printed?  
-  *B = 1000* number of simulation runs.

The data set *power_studies_results* included in MD2sample has the results for the included tests using a sample size of 200, a true type I error probability of 0.05 and two values of the parameter under the alternative, one which makes the null hypothesis true and one which makes it false. If the user wants different numbers he can run:

```{r eval=FALSE}
run.studies(study=c("NormalD2", "tD2"), 
            Continuous=TRUE, 
            param_alt=cbind(c(0.4, 0.4), c(0.7, 0.7)), 
            alpha=0.1, B=1000)
```

Say the new method can find p values without simulation:

Note that the routine should return a **named** vector with the p values.

```{r}
TSextra=list(which="pvalue", nbins=cbind(c(3,3), c(4,4)))
run.studies(study=c("NormalD2", "tD2"), 
            Continuous=TRUE, 
            TS=chiTS.cont, 
            TSextra=TSextra,
            With.p.value = TRUE, 
            B=500,
            SuppressMessages = TRUE,
            maxProcessor=1)
```

## Goodness-of-fit / twosample hybrid problem.

Consider the following situation. We have a data set $x$ and want to test whether it comes from some probability distribution *F*, that is, we have a goodness-of-fit problem. However, it is not possible to calculate probabilities from $F$, probably because this would require integration in high dimensions. We are, though, able to generate a second data set $y$ under $F$, and so can run a twosample test.

It can be shown that if the model $F$ is not fully specified but includes parameters that have to be estimated from $x$, finding p values using the permutation method leads to an extremely conservative tests, that is, the true type I error probability is much smaller than the desired one. Instead one can use a parametric bootstrap approach, that is a new data set $y$ can be generated in each simulation run.

### Example 5

Let's say we wish to test whether the data set $x$ comes from a multivariate normal distribution with unknown mean and covariance:

```{r}
f=function(a) {
    x=mvtnorm::rmvnorm(500, c(0.5, 0.5))
    y=rbind(matrix(runif(a*1000), ncol=2),
            mvtnorm::rmvnorm((1-a)*500, c(0.5,0.5)))
    list(x=x, y=y)
}    
rnull=function(dta) {
  muhat=apply(dta$x, 2, mean)
  sigmahat=cor(dta$x)
  list(x=mvtnorm::rmvnorm(nrow(dta$x), muhat, sigmahat),  
       y=mvtnorm::rmvnorm(nrow(dta$y), muhat, sigmahat))
}
```

```{r}
dta=f(0) # Null hypothesis is true
twosample_test(dta, rnull=rnull, B=BT, maxProcessor = 1)
```

```{r}
dta=f(0.2) # Null hypothesis is false
twosample_test(dta, rnull=rnull, B=BT, maxProcessor = 1)
```

Again we can find adjusted p values:

```{r eval=FALSE}
twosample_test_adjusted_pvalue(dta, rnull=rnull, 
                               B=BT, maxProcessor = 1)
```

and find the power of the tests:

```{r eval=FALSE}
twosample_power(f, c(0, 0.5), rnull=rnull, B=BP, maxProcessor=1)
```


## Methods included in MD2sample for continuous data

### Methods whose p values are found via simulation

#### Methods based on empirical distribution functions

Denote by $\mathbf{z}$ the combined data set $x_1,..,x_n,y_1,..y_m$. Let $\hat{F}$ and $\hat{G}$ be the empirical distribution functions of the two data sets, and let $\hat{H}$ be the empirical distribution function of $\mathbf{z}$  

**Kolmogorov-Smirnov test**

This classic test uses the test statistic

$$\max\{\vert \hat{F}(z_i)-\hat{G}(z_i)\vert;z_1,..,z_{n+m}\}$$

It was first proposed in [@Kolmogorov1933] and [@Smirnov1939].

**Kiuper's test**

A variant of Kolmorogov-Smirnov:

$$\max\{\hat{F}(z_i)-\hat{G}(z_i);z_1,..,z_{n+m}\}-\min\{ \hat{F}(z_i)-\hat{G}(z_i);z_1,..,z_{n+m}\}$$
This test was first discussed in [@Kuiper1960].

**Cramer-vonMises test**

$$\sum_{i=1}^{n+m} \left(\hat{F}(z_i)-\hat{G}(z_i)\right)^2$$
the extension to the two-sample problem of the Cramer-vonMises criterion is discussed in [@Anderson1962].

**Anderson-Darling test**

$$\sum_{i=1}^{n+m} \frac{\left(\hat{F}(z_i)-\hat{G}(z_i)\right)^2}{\hat{H}(z_i)(1-\hat{H}(z_i))}$$
It was first proposed in [@anderson1952].

#### Methods based on nearest neighbors

The test statistics are the average number of nearest neighbors of the $\mathbf{x}$ data set that are also from $\mathbf{x}$ plus the average number of nearest neighbors of the $\mathbf{y}$ data set that are also from $\mathbf{y}$. *NN1* uses one nearest neighbor and $NN5$ uses 5.



#### Methods based on distances between observations

We denote by $||.||$ Euclidean distance

**Aslan-Zech test**

This test discussed in [@aslanzech2005] uses the test statistic

$$
\begin{aligned}
&\frac{1}{nm}\sum_{i=1}^n \sum_{j=1}^m \log(||x_i-y_j||) -\\
&\frac{1}{n^2}\sum_{i=1}^n \sum_{i<j} \log(||x_i-x_j||) - \\
&\frac{1}{m^2}\sum_{i=1}^m \sum_{i<j} \log(||y_i-y_j||) 
\end{aligned}
$$
**Baringhaus-Franz test**

Similar to the Aslan-Zech test, it uses the test statistic

$$
\begin{aligned}
&\frac{nm}{n+m}\left[\frac{1}{nm}\sum_{i=1}^n \sum_{j=1}^m \sqrt{||x_i-y_j||} + \right.\\
&\frac{1}{n^2}\sum_{i=1}^n \sum_{i<j} \sqrt{||x_i-x_j||} -\\
&\left. \frac{1}{m^2}\sum_{i=1}^m \sum_{i<j} \sqrt{||y_i-y_j||} \right]\\
\end{aligned}
$$
and was first proposed in [@baringhaus2004].

**Biswas-Ghosh test**

Another variation of test based on Euclidean distance was discussed in [@biswas2014].

$$
\begin{aligned}
&B_{xy} = \frac{1}{nm}\sum_{i=1}^n \sum_{j=1}^m \sqrt{||x_i-y_j||} \\
&B_{xx}= \frac{2}{n(n-1)}\sum_{i=1}^n \sum_{i<j} \sqrt{||x_i-x_j||} \\
&B_{yy}=\frac{2}{m(m-1)}\sum_{i=1}^m \sum_{i<j} \sqrt{||y_i-y_j||}\\
&\left(B_{xx}-B_{xy}\right)^2+\left(B_{yy}-B_{xy}\right)^2
\end{aligned}
$$

### Methods based on kernel density estimates

**Aggregated Maximum Mean Discrepancy test**

This test is based on a measure of difference between the kernel density estimates of the two data sets, using a number of different kernel bandwidths. For details see [@gretton2023].

### Methods whose p values are found using a large sample theory


**Friedman-Rafski test**

This test is a multi-dimensional extension of the classic Wald-Wolfowitz test bases on minimum spanning trees. It was discussed in [@friedman1979].


**Simple Nearest Neighboor test**

Similar to the nearest neigboor tests described earlier, it uses only the number of nearest neighbors of the first data set that are also from the first data set. This number has a binomial distribution, and this can be used to find p values.

**Chen-Friedman tests**

These tests, discussed in [@chen2017], are implemented in the *gTests* [@gtests2017] package.

**Ball Divergence test**

A test described in [@ball2018] and implemented in the R package [@ballpackage2021].

## Methods included in MD2sample for discrete data

Implemented for discrete data are versions of the Kolmogorov-Smirnov, Kuiper, Cramer-vonMises, Anderson-Darling, nearest neighboor, Aslan-Zech, Baringhaus-Franz tests as well as a chisquare test.

# References

---
title: "Tests for MD2sample"
author: "Wolfgang Rolke"
date: "2025-03-19"
output: html_document
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, 
                      error=TRUE)
library(MD2sample)
st=function() knitr::knit_exit()
B=10
ev=c(1,1,1,1,1,1,1,1,1)
ev=ifelse(ev==1, TRUE, FALSE)
names(ev) = c("CT", "CP", "CPTS", 
              "DT", "DP",  "DPTS",
                "RunST1", "RunST2", "RunST3")
```

# Continuous Data

## Testing

```{r}
f=case.studies(1)$f
dta1=f(0)
dta2=f(0.5)

```

```{r CT, eval=ev["CT"]}
twosample_test(dta1, B=B)$p.values
twosample_test(dta2, B=B)$p.values
TSextra=list(which="statistics", nbins=rbind(c(3,3), c(4,4)))
twosample_test(dta1, TS=MD2sample::chiTS.cont, TSextra=TSextra, B=B)$p.values
twosample_test_adjusted_pvalue(dta1,B=10, maxProcessor = 1)
twosample_test_adjusted_pvalue(dta2,B=10, maxProcessor = 1)
```

## Power

```{r CPwE, eval=ev["CP"]}
twosample_power(f, c(0,0.5), B=B)
```

```{r CPTS1, eval=ev["CPTS"]}
TSextra=list(which="statistic", nbins=c(3,3))
twosample_power(f, c(0, 0.5), TS=chiTS.cont, TSextra=TSextra, B=B)
TSextra=list(which="pvalue", nbins=c(3,3))
twosample_power(f, c(0, 0.5), TS=chiTS.cont, TSextra=TSextra, B=B, With.p.value=TRUE)
```

# Discrete Data

## New Routines

## Testing

```{r}
f=case.studies(1, nbins=c(5,5))$f
dta1=f(0)
dta2=f(0.5)
```

```{r DT, eval=ev["DT"]}
twosample_test(dta1, B=B)$p.values
twosample_test(dta2, B=B)$p.values
TSextra=list(which="statistic")
twosample_test(dta1, TS=MD2sample::chiTS.disc, TSextra=TSextra, B=B)$p.values
twosample_test_adjusted_pvalue(dta1,B=10, maxProcessor = 1)
```

## Power

```{r DPwE, eval=ev["DP"]}
twosample_power(f, c(0,0.5),  B=B)
TSextra=list(which="statistic")
twosample_power(f, c(0,0.5), TS=chiTS.disc, TSextra=TSextra, B=B)
TSextra=list(which="pvalue")
twosample_power(f, c(0,0.5), TS=chiTS.disc, TSextra=TSextra, B=B, With.p.value=TRUE)
```


# Run Studies

```{r}
cases=MD2sample::case.studies(ReturnCaseNames = TRUE)
```

```{r eval=ev["RunST1"]}
run.studies(Continuous,TS1, "NormalD2", B=10)
if(Continuous) run.studies(Continuous,TS2, c("NormalD2", "tD2"), With.p.value=TRUE, B=10)
```

```{r eval=ev["RunST2"]}
run.studies(Continuous, TS1, "NormalD2", param_alt=0.8, alpha=0.1, nsample=100, B=10)
if(Continuous)
   run.studies(Continuous, TS2, c("NormalD2", "tD2"), param_alt=list(c(0,0.8),c(0,0.8)),
            With.p.value=TRUE, alpha=0.1, nsample=100, B=10)
```

```{r eval=ev["RunST3"]}
p=readRDS("power_studies_results.RDS")[[3]]
run.studies(Continuous, param_alt=p, 
            alpha=0.1, B=10, maxProcessor = 1)
```

---
title: "Case Studies"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Case Studies}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(MD2sample)
cases=MD2sample::case.studies(ReturnCaseNames=TRUE)
```

```{r}
multiple.graphs = function (px, py,xname=" ") {
  egg::ggarrange(px, py, ncol = 2, nrow = 1)
} 
```

This vignette lists the case studies included in the package:

## Dim=2, equal marginals

```{r}
for(i in seq_along(cases)) {
   if(!endsWith(cases[i], "D2")) next
   tmp=case.studies(i, 2000)
   print(cases[i])
   p=tmp$param_alt[2]
   if(startsWith(cases[i],"Dalitz")) p=10*p
   dta=tmp$f(p)
   dtax=data.frame(x=dta$x[,1], y=dta$x[,2])
   dtay=data.frame(x=dta$y[,1], y=dta$y[,2])
   px=ggplot2::ggplot(dtax, ggplot2::aes(x=x, y=y))+
     ggplot2::geom_point(color="blue", alpha=0.5, size=0.5)+
     ggplot2::labs(x=expression(x[1]), y=expression(x[2])) 
     
   py=ggplot2::ggplot(dtay, ggplot2::aes(x=x, y=y))+
     ggplot2::geom_point(color="red", alpha=0.5)+
     ggplot2::labs(x=expression(x[1]), y=expression(x[2])) 
  multiple.graphs(px, py, cases[i])
}
```


## Dim=2, unequal marginals

```{r}
for(i in seq_along(cases)) {
   if(!endsWith(cases[i], "M")) next
   tmp=case.studies(i, 2000)
   print(cases[i])
   p=tmp$param_alt[2]
   if(startsWith(cases[i],"Dalitz")) p=10*p
   dta=tmp$f(p)
   dtax=data.frame(x=dta$x[,1], y=dta$x[,2])
   dtay=data.frame(x=dta$y[,1], y=dta$y[,2])
   px=ggplot2::ggplot(dtax, ggplot2::aes(x=x, y=y))+
     ggplot2::geom_point(color="blue", alpha=0.5, size=0.5)+
     ggplot2::labs(x=expression(x[1]), y=expression(x[2])) 
     
   py=ggplot2::ggplot(dtay, ggplot2::aes(x=x, y=y))+
     ggplot2::geom_point(color="red", alpha=0.5)+
     ggplot2::labs(x=expression(x[1]), y=expression(x[2])) 
  multiple.graphs(px, py, cases[i])
}
```


## Dim=5, equal marginals


-   NormalD5:  multivariate normal distribution with equal marginals.  
-   tD5:  multivariate t distribution with 5 degrees of freedom and equal marginals.                 
- FrankD5: Frank cupola.  
- ClaytonD5: Clayton cupola.  
- GumbelD5: Gumbel copula.  
- JoeD5: Joe cupola.  
- UniformFrankD5: mixture of uniform and Frank cupola.  
- FrankClaytonD5: mixture of Frank and Clayton cupolas.  
- FrankJoeD5: mixture of Frank and Joe cupolas.           


## Dim=5, unequal marginals

- UniformExponentialM5: Exponential distributions.  
- FrankExponentialM5: Frank cupola with exponential marginals.  
- FrankLinearM5: Frank cupola with linear marginals.  
- FrankNormalM5: Frank cupola with linear marginals.  
- ClaytonExponentialM5: Clayton cupola with exponential marginals.  
- ClaytonLinearM5: Clayton cupola with linear marginals.  
- ClaytonNormalM5: Clayton cupola with linear marginals.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chisqTS.R
\name{chiTS.cont}
\alias{chiTS.cont}
\title{Chi-square test for continuous data, example of user supplied function}
\usage{
chiTS.cont(x, y, TSextra)
}
\arguments{
\item{x}{a matrix of numbers}

\item{y}{a matrix of numbers}

\item{TSextra}{list with some info}
}
\value{
either a test statistic or a p value
}
\description{
This function does the chi square two sample test for continuous data in two dimensions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{powerC}
\alias{powerC}
\title{Find the power of various continuous tests via simutation or permutation.}
\usage{
powerC(rxy, xparam, yparam, TS, typeTS, TSextra, B = 1000L)
}
\arguments{
\item{rxy}{a function that generates x and y data.}

\item{xparam}{arguments for r1.}

\item{yparam}{arguments for r2.}

\item{TS}{routine to calculate test statistics for non-chi-square tests}

\item{typeTS}{indicator for type of test statistics}

\item{TSextra}{additional info passed to TS, if necessary}

\item{B}{=1000 number of simulation runs}
}
\value{
A list values of test statistics
}
\description{
Find the power of various continuous tests via simutation or permutation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twosample_power.R
\name{twosample_power}
\alias{twosample_power}
\title{Power Estimation for Multivariate Two-Sample Tests}
\usage{
twosample_power(
  f,
  ...,
  TS,
  TSextra,
  alpha = 0.05,
  B = 1000,
  nbins = c(5, 5),
  minexpcount = 5,
  Ranges = matrix(c(-Inf, Inf, -Inf, Inf), 2, 2),
  samplingmethod = "Binomial",
  rnull,
  With.p.value = FALSE,
  DoTransform = TRUE,
  SuppressMessages = FALSE,
  LargeSampleOnly = FALSE,
  maxProcessor,
  doMethods = "all"
)
}
\arguments{
\item{f}{function to generate a list with data sets x and y for continuous data or
a matrix with columns vals_x, vals_y, x and y for discrete data.}

\item{...}{additional arguments passed to f, up to 2.}

\item{TS}{routine to calculate test statistics for new tests.}

\item{TSextra}{additional info passed to TS, if necessary.}

\item{alpha}{=0.05, the type I error probability of the hypothesis test.}

\item{B}{=1000, number of simulation runs.}

\item{nbins}{=c(5, 5), number of bins for chi square test if Dim=2.}

\item{minexpcount}{=5, lowest required count for chi-square test.}

\item{Ranges}{=matrix(c(-Inf, Inf, -Inf, Inf),2,2), a 2x2 matrix with lower and upper bounds.}

\item{samplingmethod}{="Binomial" for Binomial sampling or "independence" for independence
sampling in the discrete data case.}

\item{rnull}{function to generate new data sets for parametric bootstrap.}

\item{With.p.value}{=FALSE, does user supplied routine return p values?}

\item{DoTransform}{=TRUE, should data be transformed to  to unit hypercube?}

\item{SuppressMessages}{=FALSE, should messages be printed?}

\item{LargeSampleOnly}{=FALSE, should only methods with large sample theories be run?}

\item{maxProcessor}{number of cores to use. If missing the number of physical cores-1
is used. If set to 1 no parallel processing is done.}

\item{doMethods}{="all", which methods should be included?}
}
\value{
A numeric matrix or vector of power values.
}
\description{
Estimate the power of various two sample tests using Rcpp and parallel computing.
}
\details{
For details consult vignette("MD2sample","MD2sample")
}
\examples{
#Note that the resulting power estimates are meaningless because
#of the extremely low number of simulation runs B, required because of CRAN timing rule
#
#Power of tests when one data set comes from a standard normal multivariate distribution function
#and the other data set from a multivariate normal with correlation
#number of simulation runs is ridiculously small because of CRAN submission rules
f=function(a=0) {
 S=diag(2) 
 x=mvtnorm::rmvnorm(100, sigma = S)
 S[1,2]=a
 S[2,1]=a
 y=mvtnorm::rmvnorm(120, sigma = S)
 list(x=x, y=y)
}
twosample_power(f, c(0, 0.5), B=10, maxProcessor=1)
#Power of use supplied test. Example is a (included) chi-square test:
TSextra=list(which="statistics", nbins=rbind(c(3,3), c(4,4)))
twosample_power(f, c(0, 0.5), TS=chiTS.cont, TSextra=TSextra, B=10, maxProcessor=1)
#Same example, but this time the user supplied routine calculates p values:
TSextra=list(which="pvalues", nbins=c(4,4))
twosample_power(f, c(0, 0.5), TS=chiTS.cont, TSextra=TSextra, B=10, 
             With.p.value=TRUE, maxProcessor=1)
#Example for discrete data
g=function(p1, p2) {
  x = table(sample(1:4, size=1000, replace = TRUE))
  y = table(sample(1:4, size=500, replace = TRUE, prob=c(p1,p2,1,1)))
  cbind(vals_x=rep(1:2,2),  vals_y=rep(1:2, each=2), x=x, y=y)
}  
twosample_power(g, 1.5, 1.6, B=10, maxProcessor=1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/change.marginals.R
\name{change.marginals}
\alias{change.marginals}
\title{Change Marginals of 2D Data}
\usage{
change.marginals(dta, which, theta)
}
\arguments{
\item{dta}{a list with two data sets}

\item{which}{method for modifying the data sets.}

\item{theta}{how much to modify}
}
\value{
a list of functions
}
\description{
This function modifies the marginals of some data set
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{gen_sim_data}
\alias{gen_sim_data}
\title{simulate new data}
\usage{
gen_sim_data(dta, TSextra)
}
\arguments{
\item{dta}{data set}

\item{TSextra}{extra stuff}
}
\value{
A list of permuted vectors
}
\description{
simulate new data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/case.studies.R
\name{case.studies}
\alias{case.studies}
\title{Create case studies}
\usage{
case.studies(
  which,
  n = 200,
  nx = n,
  ny = n,
  nbins = -1,
  Ranges = matrix(c(-Inf, Inf, -Inf, Inf), 2, 2),
  ReturnCaseNames = FALSE
)
}
\arguments{
\item{which}{name of the case study.}

\item{n}{=200, sample size for both data sets}

\item{nx}{=n, sample size for first data sets}

\item{ny}{=n, sample size for second data sets}

\item{nbins}{=-1, number of bins for chi-square test, 2D only}

\item{Ranges}{= matrix(c(-Inf, Inf, -Inf, Inf), 2, 2), ranges of variables}

\item{ReturnCaseNames}{=FALSE, should list of case studies be returned?}
}
\value{
a list of functions and vectors
}
\description{
This function creates the functions needed to run the various case studies.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{mmdagg}
\alias{mmdagg}
\title{Find test statistic for MMD test}
\usage{
mmdagg(x, y)
}
\arguments{
\item{x}{A matrix}

\item{y}{A matrix}
}
\value{
A numeric vector with test statistics
}
\description{
Find test statistic for MMD test
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{calcTS}
\alias{calcTS}
\title{This function calculates the test statistics}
\usage{
calcTS(dta, TS, typeTS, TSextra)
}
\arguments{
\item{dta}{data set}

\item{TS}{routine}

\item{typeTS}{format of TS}

\item{TSextra}{list passed to TS function}
}
\value{
A vector of numbers
}
\description{
This function calculates the test statistics
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{TS_disc}
\alias{TS_disc}
\title{Find test statistics for discrete data}
\usage{
TS_disc(x, y, vx, vy, TSextra)
}
\arguments{
\item{x}{a vector of integers.}

\item{y}{a vector of integers.}

\item{vx}{a vector of real.}

\item{vy}{a vector of real.}

\item{TSextra}{list with extra infos}
}
\value{
A numeric vector with test statistics
}
\description{
Find test statistics for discrete data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{power_studies_results}
\alias{power_studies_results}
\title{power_studies_results}
\format{
\subsection{'power_studies_results'}{

A list of matrices with powers
}
}
\usage{
power_studies_results
}
\description{
the results of the included power studies
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{find_dist}
\alias{find_dist}
\title{find pairwise Euclidean distances}
\usage{
find_dist(dta)
}
\arguments{
\item{dta}{A list of matrices}
}
\value{
A  matrix
}
\description{
find pairwise Euclidean distances
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power_pvals.R
\name{power_pvals}
\alias{power_pvals}
\title{This function finds the power for tests that find their own p values}
\usage{
power_pvals(
  rxy,
  avals,
  bvals,
  TS,
  typeTS,
  TSextra = list(a = 0),
  nbins = c(5, 5),
  minexpcount = 5,
  Ranges = matrix(c(-Inf, Inf, -Inf, Inf), 2, 2),
  alpha = 0.05,
  B = 1000
)
}
\arguments{
\item{rxy}{a function to generate data}

\item{avals}{vector of parameter values}

\item{bvals}{vector of parameter values}

\item{TS}{routine that runs the test and returns p values}

\item{typeTS}{type of problem, continuous or discrete}

\item{TSextra}{=list(a=0) a list of things passed to TS, if needed}

\item{nbins}{=c(5, 5), bins for chi square tests.}

\item{minexpcount}{=5, lowest required count for chi-square test}

\item{Ranges}{=matrix(c(-Inf, Inf, -Inf, Inf),2,2) a 2x2 matrix with lower and upper bounds}

\item{alpha}{=0.05 type I error probability of test}

\item{B}{=1000 number of simulation runs}
}
\value{
A matrix of power values
}
\description{
This function finds the power for tests that find their own p values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{TS_cont}
\alias{TS_cont}
\title{Find test statistics for continuous data}
\usage{
TS_cont(x, y, TSextra)
}
\arguments{
\item{x}{A matrix.}

\item{y}{A matrix}

\item{TSextra}{list with some helper functions}
}
\value{
A numeric vector with test statistics
}
\description{
Find test statistics for continuous data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MD2sample-package.R
\docType{package}
\name{MD2sample-package}
\alias{MD2sample}
\alias{MD2sample-package}
\title{MD2sample: Various Methods for the Two Sample Problem in D>1 Dimensions}
\description{
The routine twosample_test() in this package runs the two-sample test using various test statistic for multivariate data. The user can also run several tests and then find a p value adjusted for simultaneous inference. The p values are found via permutation or via the parametric bootstrap. The routine twosample_power() allows the estimation of the power of the tests. The routine run.studies() allows a user to quickly study the power of a new method and how it compares to those included in the package. For details of the methods and references see the included vignettes.
}
\author{
\strong{Maintainer}: Wolfgang Rolke \email{wolfgang.rolke@upr.edu} (\href{https://orcid.org/0000-0002-3514-726X}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chisq2D_test_cont.R
\name{chisq2D_test_cont}
\alias{chisq2D_test_cont}
\title{This function does the chi square two sample test for continuous data in two dimensions}
\usage{
chisq2D_test_cont(
  dta_x,
  dta_y,
  Ranges = matrix(c(-Inf, Inf, -Inf, Inf), 2, 2),
  nbins = c(5, 5),
  minexpcount = 5,
  SuppressMessages = FALSE
)
}
\arguments{
\item{dta_x}{a matrix of numbers}

\item{dta_y}{a matrix of numbers}

\item{Ranges}{=matrix(c(-Inf, Inf, -Inf, Inf),2,2) a 2x2 matrix with lower and upper bounds}

\item{nbins}{=c(5,5) number of bins in x and y direction}

\item{minexpcount}{=5 minimum counts required per bin}

\item{SuppressMessages}{=FALSE, print informative messages?}
}
\value{
a list with statistics and p values
}
\description{
This function does the chi square two sample test for continuous data in two dimensions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{gen_cont}
\alias{gen_cont}
\title{simulate continuous data}
\usage{
gen_cont(x, y, TSextra)
}
\arguments{
\item{x}{first data set}

\item{y}{second data set}

\item{TSextra}{extra stuff}
}
\value{
A list of permuted vectors
}
\description{
simulate continuous data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{Rank}
\alias{Rank}
\title{equivalent to R command rank}
\usage{
Rank(x)
}
\arguments{
\item{x}{numeric vector}
}
\value{
integer vector
}
\description{
equivalent to R command rank
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{gen_disc}
\alias{gen_disc}
\title{simulate new discrete data}
\usage{
gen_disc(x, y, vals_x, vals_y, TSextra)
}
\arguments{
\item{x}{first data set, counts}

\item{y}{second data set, counts}

\item{vals_x}{values of discrete random variable x}

\item{vals_y}{values of discrete random variable y}

\item{TSextra}{extra stuff}
}
\value{
A list of permuted vectors
}
\description{
simulate new discrete data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FR.test.R
\name{FR.test}
\alias{FR.test}
\title{Friedman-Rafsky (FR) test}
\usage{
FR.test(
  samp1,
  samp2,
  use.cosine = FALSE,
  binary = FALSE,
  binary.cutoff = 2,
  plot.MST = FALSE,
  col = c("#F0E442", "#56B4E9"),
  label.names = c("Sample 1", "Sample 2"),
  vertex.size = 5,
  edge.width = 1,
  ...
)
}
\arguments{
\item{samp1}{Numeric matrix or data frame for Sample 1. Rows are multivariate dimensions, and columns are samples. E.g. gene by cell.}

\item{samp2}{Numeric matrix or data frame for Sample 2.}

\item{use.cosine}{An option if to use cosine distance. Logical variable. By default (\code{FALSE}),
Euclidean distance is used.}

\item{binary}{An option if to use binary values. Logical variable. Default: \code{FALSE}. If \code{TRUE},
use \code{binary.cutoff} to dichotomize \code{samp1} and \code{samp2}.}

\item{binary.cutoff}{Numeric value for binary cutoff. Binary value = 1 if greater than \code{binary.cutoff}, 0 otherwise. Default: \code{2}.}

\item{plot.MST}{Boolean variable indicating if to plot the minimum spanning tree (MST). Default: \code{FALSE}.}

\item{col}{Character vector of length two for customized colors of the nodes in MST. Default: \code{c("#F0E442", "#56B4E9")}.}

\item{label.names}{Character vector of length two for customized names of the two samples. Default: \code{c("Sample 1","Sample 2")}.}

\item{vertex.size, edge.width, ...}{Additional plotting parammeters passed to \code{\link[igraph]{plot.igraph}}. Default: \code{vertex.size=5, edge.width=1}.}
}
\value{
Test statistics and p-values.
\item{runs}{Total number of subtrees.}
\item{runs.samp1}{Number of subtrees of Sample 1.}
\item{runs.samp2}{Number of subtrees of Sample 2.}
\item{stat}{The standardized FR statistic.}
\item{p.value}{P-value of the FR test.}
}
\description{
FR test is a multivariate generalization of nonparametric two-sample test. This function is an implementation
with customized options, including a visualization of the minimum spanning tree (MST).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{xysort}
\alias{xysort}
\title{sort vector y by values in vector x}
\usage{
xysort(y, x)
}
\arguments{
\item{y}{numeric vector}

\item{x}{numeric vector}
}
\value{
numeric vector
}
\description{
sort vector y by values in vector x
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twosample_test_adjusted_pvalue.R
\name{twosample_test_adjusted_pvalue}
\alias{twosample_test_adjusted_pvalue}
\title{Adjusted p values}
\usage{
twosample_test_adjusted_pvalue(
  x,
  y,
  vals_x = NA,
  vals_y = NA,
  B = c(5000, 1000),
  nbins = c(5, 5),
  minexpcount = 5,
  samplingmethod = "Binomial",
  Ranges = matrix(c(-Inf, Inf, -Inf, Inf), 2, 2),
  DoTransform = TRUE,
  rnull,
  SuppressMessages = FALSE,
  maxProcessor,
  doMethods
)
}
\arguments{
\item{x}{Continuous data: either a matrix of numbers, or a list with two matrices called x and y.
if it is a matrix Observations are in different rows.
Discrete data: a vector of counts or a matrix with columns named vals_x, vals_y, x and y.}

\item{y}{a matrix of numbers if data if data is continuous or a vector of counts  if data is discrete.}

\item{vals_x}{=NA, a vector of values for discrete random variable, or NA if data is continuous.}

\item{vals_y}{=NA, a vector of values for discrete random variable, or NA if data is continuous.}

\item{B}{=c(5000, 1000), number of simulation runs for permutation test and for estimation
of the empirical distribution function.}

\item{nbins}{=c(5, 5), number of bins for chi square tests (2D only).}

\item{minexpcount}{= 5, minimum required expected counts for chi-square tests.}

\item{samplingmethod}{="Binomial" or "independence" for discrete data.}

\item{Ranges}{=matrix(c(-Inf, Inf, -Inf, Inf),2,2) a 2x2 matrix with lower and upper bounds.}

\item{DoTransform}{=TRUE, should data be transformed to interval (0,1)?}

\item{rnull}{routine for parametric bootstrap.}

\item{SuppressMessages}{= FALSE, print informative messages?}

\item{maxProcessor}{number of cores for parallel processing.}

\item{doMethods}{Which methods should be included? If missing a small number of methods that generally have good power are used.}
}
\value{
NULL, results are printed out.
}
\description{
This function runs a number of two sample tests using Rcpp and parallel computing and then finds the correct p value for the combined tests.
}
\details{
For details consult the vignette("MD2sample","MD2sample")
}
\examples{
#Note that the number of simulation runs B is very small to
#satisfy CRAN's run time constraints. 
#Two continuous data sets from a multivariate normal:
x = mvtnorm::rmvnorm(100, c(0,0))
y = mvtnorm::rmvnorm(120, c(0,0))
twosample_test_adjusted_pvalue(x, y, maxProcessor=1, B=20)
#Two discrete data sets from some distribution:
x = table(sample(1:4, size=1000, replace = TRUE))
y = table(sample(1:4, size=500, replace = TRUE, prob=c(1, 1.5, 1, 1)))
twosample_test_adjusted_pvalue(x, y, rep(1:2,2), rep(1:2, each=2), maxProcessor=1, B=20)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test_methods.R
\name{test_methods}
\alias{test_methods}
\title{This function checks whether the correct methods have been requested}
\usage{
test_methods(doMethods, Continuous, ReturnMethodNames = FALSE)
}
\arguments{
\item{doMethods}{="all" Which methods should be included?}

\item{Continuous}{is data continuous}

\item{ReturnMethodNames}{= FALSE should names of methods be returned?}
}
\value{
TRUE/FALSE or a character vector
}
\description{
This function checks whether the correct methods have been requested
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chisq2D_test_disc.R
\name{chisq2D_test_disc}
\alias{chisq2D_test_disc}
\title{This function does the chi square two sample test for continuous data in two dimensions}
\usage{
chisq2D_test_disc(dta, minexpcount = 5)
}
\arguments{
\item{dta}{a list with values and counts}

\item{minexpcount}{=5 minimum counts required per bin}
}
\value{
a list with statistics and p values
}
\description{
This function does the chi square two sample test for continuous data in two dimensions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{testC}
\alias{testC}
\title{run tests using either simulation or permutation.}
\usage{
testC(dta, TS, typeTS, TSextra, B = 5000L)
}
\arguments{
\item{dta}{a list with the data}

\item{TS}{routine to calculate test statistics for non-chi-square tests}

\item{typeTS}{type of a test statistic}

\item{TSextra}{additional info passed to TS, if necessary}

\item{B}{=5000, number of simulation runs.}
}
\value{
A list with test statistics and p values
}
\description{
run tests using either simulation or permutation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rDalitz.R
\name{rDalitz}
\alias{rDalitz}
\title{This function creates data for Dalitz plot}
\usage{
rDalitz(nyadd = 0, which = 1, addon, n = 200, nx = n, ny = n)
}
\arguments{
\item{nyadd}{=0}

\item{which}{type of Daliz plot.}

\item{addon}{what to add to uniform}

\item{n}{=200, sample size of both data sets.}

\item{nx}{=n, sample size of x data set.}

\item{ny}{=n, sample size of y data set.}
}
\value{
a list of functions
}
\description{
This function creates data for Dalitz plot
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signif.digits.R
\name{signif.digits}
\alias{signif.digits}
\title{This function does some rounding to nice numbers}
\usage{
\method{signif}{digits}(x, d = 4)
}
\arguments{
\item{x}{a list of two vectors}

\item{d}{=4 number of digits to round to}
}
\value{
A list with rounded vectors
}
\description{
This function does some rounding to nice numbers
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TS_cont_pval.R
\name{TS_cont_pval}
\alias{TS_cont_pval}
\title{This function runs a number of two sample tests which find their own p value.}
\usage{
TS_cont_pval(x, y)
}
\arguments{
\item{x}{a matrix of numbers if data is continuous or of counts  if data is discrete.}

\item{y}{a matrix of numbers if data is continuous or of counts  if data is discrete.}
}
\value{
A list of two numeric vectors, the test statistics and the p values.
}
\description{
This function runs a number of two sample tests which find their own p value.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run.studies.R
\name{run.studies}
\alias{run.studies}
\title{Benchmarking for Multivariate Two-Sample Tests}
\usage{
run.studies(
  Continuous = TRUE,
  study,
  TS,
  TSextra,
  With.p.value = FALSE,
  nsample = 200,
  alpha = 0.05,
  param_alt,
  SuppressMessages = FALSE,
  B = 1000,
  maxProcessor
)
}
\arguments{
\item{Continuous}{=TRUE, run cases for continuous data.}

\item{study}{either the name of the study, or its number in the list. If missing all the studies are run.}

\item{TS}{routine to calculate new test statistics.}

\item{TSextra}{list passed to TS (optional).}

\item{With.p.value}{=FALSE, does user supplied routine return p values?}

\item{nsample}{= 200, desired sample size. 200 is used in included case studies.}

\item{alpha}{=0.05,  type I error probability of tests. 0.05 is used in included case studies.}

\item{param_alt}{(list of) values of parameter under the alternative hypothesis.
If missing included values are used.}

\item{SuppressMessages}{=FALSE, should informative messages be printed?}

\item{B}{= 1000, number of simulation runs.}

\item{maxProcessor}{number of cores to use. If missing the number of physical cores-1
is used. If set to 1 no parallel processing is done.}
}
\value{
A (list of ) matrices of p.values.
}
\description{
This function runs the case studies included in the package.
}
\details{
For details consult vignette(package="MD2sample")
}
\examples{
#The new test is a (included) chi square test:
TSextra=list(which="pval", nbins=rbind(c(3,3), c(4,4)))
run.studies(Continuous=TRUE, study=c("NormalD2", "tD2"), 
          TS=MD2sample::chiTS.cont, TSextra=TSextra, 
          With.p.value=TRUE, B=100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twosample_test.R
\name{twosample_test}
\alias{twosample_test}
\title{Multivariate Two-Sample Tests}
\usage{
twosample_test(
  x,
  y,
  vals_x = NA,
  vals_y = NA,
  TS,
  TSextra,
  B = 5000,
  nbins = c(5, 5),
  minexpcount = 5,
  Ranges = matrix(c(-Inf, Inf, -Inf, Inf), 2, 2),
  DoTransform = TRUE,
  samplingmethod = "Binomial",
  rnull,
  SuppressMessages = FALSE,
  LargeSampleOnly = FALSE,
  maxProcessor,
  doMethods = "all"
)
}
\arguments{
\item{x}{Continuous data: either a matrix of numbers, or a list with two matrices called x and y.
if it is a matrix Observations are in different rows.
Discrete data: a vector of counts or a matrix with columns named vals_x, vals_y, x and y.}

\item{y}{a matrix of numbers if data is continuous or a vector of counts  if data is discrete.}

\item{vals_x}{=NA, a vector of values for discrete random variables, or NA if data is continuous.}

\item{vals_y}{=NA, a vector of values for discrete random variables, or NA if data is continuous.}

\item{TS}{user supplied routine to calculate test statistics for new tests.}

\item{TSextra}{(optional) additional info passed to TS, if necessary.}

\item{B}{=5000, number of simulation runs for permutation test.}

\item{nbins}{=c(5,5), for chi square tests (2D only).}

\item{minexpcount}{=5, lowest required count for chi-square test (2D only).}

\item{Ranges}{=matrix(c(-Inf, Inf, -Inf, Inf),2,2), a 2x2 matrix with lower and upper bounds (2D only).}

\item{DoTransform}{=TRUE, should data be transformed to unit hypercube?}

\item{samplingmethod}{="Binomial" for Binomial sampling or "independence" for independence sampling.}

\item{rnull}{function to generate new data sets for simulation as an alternative to the permutation method.}

\item{SuppressMessages}{=FALSE, should informative messages be printed?}

\item{LargeSampleOnly}{=FALSE, should only methods with large sample theories be run?}

\item{maxProcessor}{number of cores to use. If missing the number of physical cores-1
is used. If set to 1 no parallel processing is done.}

\item{doMethods}{="all", Which methods should be included?}
}
\value{
A list of two numeric vectors, the test statistics and the p values.
}
\description{
This function runs a number of two sample tests using Rcpp and parallel computing.
}
\details{
For details consult vignette("MD2sample","MD2sample")
}
\examples{
#Two continuous data sets from a multivariate normal:
x = mvtnorm::rmvnorm(100, c(0,0))
y = mvtnorm::rmvnorm(120, c(0,0))
twosample_test(x, y, B=100, maxProcessor=1)
#Using a new test, this one is an (included) chi square test. 
#Also enter data as a list:
TSextra=list(which="statistics", nbins=rbind(c(3,3), c(4,4)))
dta=list(x=x, y=y)
twosample_test(dta, TS=chiTS.cont, TSextra=TSextra, B=100, maxProcessor=1)
#Two discrete data sets from some distribution:
x = table(sample(1:4, size=1000, replace = TRUE))
y = table(sample(1:4, size=1000, replace = TRUE, prob=c(1,2,1,1)))
vals_x=rep(1:2,2)
vals_y=rep(1:2, each=2)
twosample_test(x, y, vals_x, vals_y, B=100, maxProcessor=1)
#Run a discrete chi square test and enter the data as a matrix:
TSextra=list(which="statistics")
dta=cbind(x=x, y=y, vals_x=vals_x, vals_y=vals_y)
twosample_test(dta, TS=chiTS.disc, TSextra=TSextra, B=100, maxProcessor=1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timecheck.R
\name{timecheck}
\alias{timecheck}
\title{test function}
\usage{
timecheck(dta, TS, typeTS, TSextra)
}
\arguments{
\item{dta}{data set}

\item{TS}{test statistics}

\item{typeTS}{format of TS}

\item{TSextra}{additional info TS}
}
\value{
Mean computation time
}
\description{
test function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twosample_test_adjusted_pvalue.R
\name{simTS}
\alias{simTS}
\title{Helper function to find test statistics of simulated data.}
\usage{
simTS(dta, TS, typeTS, TSextra, B)
}
\arguments{
\item{dta}{a list}

\item{TS}{test statistic routine}

\item{typeTS}{type of routine}

\item{TSextra}{a list}

\item{B}{number of simulation runs}
}
\value{
a matrix
}
\description{
Helper function to find test statistics of simulated data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edge.tests.R
\name{edge.tests}
\alias{edge.tests}
\title{This function does the tests by Chen and Friedman}
\usage{
edge.tests(x, y)
}
\arguments{
\item{x}{a matrix of numbers}

\item{y}{a matrix of numbers}
}
\value{
a list with statistics and p values
}
\description{
This function does the tests by Chen and Friedman
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bincounterR.R
\name{bincounterR}
\alias{bincounterR}
\title{Count Events in Bins}
\usage{
bincounterR(
  x,
  y,
  nbins = c(5, 5),
  Ranges = matrix(c(-Inf, Inf, -Inf, Inf), 2, 2)
)
}
\arguments{
\item{x}{a matrix of numbers or a list}

\item{y}{a matrix of numbers}

\item{nbins}{=c(5,5) number of bins in x and in y direction}

\item{Ranges}{=matrix(c(-Inf, Inf, -Inf, Inf),2,2) a 2x2 matrix with lower and upper bounds}
}
\value{
a list counts and vals
}
\description{
This function discretizes continuous data in two dimensions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chisqTS.R
\name{chiTS.disc}
\alias{chiTS.disc}
\title{Chi-square test for discrete data, example of user supplied function}
\usage{
chiTS.disc(x, y, vals_x, vals_y, TSextra)
}
\arguments{
\item{x}{a vector with counts}

\item{y}{a vector with counts}

\item{vals_x}{a vector with points}

\item{vals_y}{a vector with points}

\item{TSextra}{list with some info}
}
\value{
either a test statistic or a p value
}
\description{
This function does the chi square two sample test for discrete data in two dimensions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{transform01}
\alias{transform01}
\title{transform data to interval (0,1).}
\usage{
transform01(dta)
}
\arguments{
\item{dta}{a list.}
}
\value{
a list
}
\description{
transform data to interval (0,1).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{mdecdf}
\alias{mdecdf}
\title{Finds the empirical distribution function}
\usage{
mdecdf(dta, pts)
}
\arguments{
\item{dta}{a matrix of data points}

\item{pts}{a matrix of evaluation points}
}
\value{
a numeric vector
}
\description{
Finds the empirical distribution function
}
\keyword{internal}
